/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.internal.ReportsView");

dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.SideNavigationLayout");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.reports.web.app.internal.ReportsController");
dojo.require("com.ibm.team.reports.web.app.view.navigation.AdminNavigator");
dojo.require("com.ibm.team.reports.web.app.view.navigation.IncludeArchived");
dojo.require("com.ibm.team.reports.web.app.view.navigation.ReportsNavigator");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.app.view.admin.AdminViewer");
dojo.require("com.ibm.team.reports.web.app.view.report.ReportViewer");
dojo.require("com.ibm.team.reports.web.app.view.report.NewReportEditor");
dojo.require("com.ibm.team.reports.web.app.view.navigation.WorkItemsNavigator");
dojo.require("com.ibm.team.reports.web.app.view.report.ReportsWelcome");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

(function() { // module container

var session	= com.ibm.team.repository.web.client.session;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var WorkItemsNavigator = com.ibm.team.reports.web.app.view.navigation.WorkItemsNavigator;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ActionRegistry = null; // Initialize later
var newReportURI = "#action=com.ibm.team.reports.newReport";

/**
 * Main perspective view for the Reports perspective.
 */
dojo.declare("com.ibm.team.reports.web.app.internal.ReportsView", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/internal/templates/ReportsView.html"),

		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
		},
		
		postCreate: function() {
			com.ibm.team.reports.web.app.internal.ReportsView.superclass.postCreate.apply(this);

		 	this.controller = new com.ibm.team.reports.web.app.internal.ReportsController(this);
		    this._mainView = new net.jazz.ajax.ui.SideNavigationLayout({}, this._appContainer);
		    this.parentController = this._mainView.controller;
		    
		 	this.controller.parentController = this._mainView.controller; 
		 	this.controller.prototype = this._mainView.controller;
		 	
		 	// Lazy initialize ActionRegistry
		 	ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		 	
			ActionRegistry.registerAction("com.ibm.team.reports.defaultAction", this.getController(), "defaultAction");
			ActionRegistry.registerAction("com.ibm.team.reports.viewQuery", this.getController(), "viewQuery");
			ActionRegistry.registerAction("com.ibm.team.reports.viewReport", this.getController(), "viewReport");
			ActionRegistry.registerAction("com.ibm.team.reports.getMyQueries", this.getController(), "getMyQueries");
			ActionRegistry.registerAction("com.ibm.team.reports.getSharedQueries", this.getController(), "getSharedQueries");
			ActionRegistry.registerAction("com.ibm.team.reports.admin", this.getController(), "admin");
		 	ActionRegistry.registerAction("com.ibm.team.reports.viewWorkItems", this.getController(), "viewWorkItems");
		 	ActionRegistry.registerAction("com.ibm.team.reports.newReport", this.getController(), "newReport");
		 	
		    this._canDeployQuery = false;
		    this._canManageFolder = false;
		    // The number of asnyc calls below we need to wait for
		    this._numberOfPendingResponses = 3;
			var responseHandler = new ServiceResponseHandler(
				this,
				"_handleCanDeployQuery",
				"_remoteError");
			ReportsClient.getCanDeployQuery(responseHandler);
			responseHandler = new ServiceResponseHandler(
				this,
				"_handleCanManageFolder",
				"_remoteError");
			ReportsClient.getCanManageFolder(responseHandler);
			responseHandler = new ServiceResponseHandler(
				this,
				"_handleGetDefaultQueryDescriptor",
				"_remoteError");
			ReportsClient.getDefaultQueryDescriptor(responseHandler);
			
		},
		
		/**
		 * Framework API for hyperlinks and back/forward.
		 */
		getController: function() {
			return this.controller;
		},
		
		_handleGetDefaultQueryDescriptor: function(result) {
			this.controller.setDefaultQuery(result);
			this._numberOfPendingResponses--;
			if (this._numberOfPendingResponses > 0) {
				return;
			}
			this._initializeNavigation();
		},
		
		_handleCanDeployQuery: function(result) {
			if (result == "true") {
				this._canDeployQuery = true;
			}
			this._numberOfPendingResponses--;
			if (this._numberOfPendingResponses > 0) {
				return;
			}
			this._initializeNavigation();
		},
		
		_handleCanManageFolder: function(result) {
			if (result == "true") {
				this._canManageFolder = true;
			}
			this._numberOfPendingResponses--;
			if (this._numberOfPendingResponses > 0) {
				return;
			}
			this._initializeNavigation();		
		},
		
		/**
		 * Initializes the left navigation section.
		 */
		_initializeNavigation: function() {
			if (!com.ibm.team.repository.web.client.session.isGuest() && this._canDeployQuery) {
				this._newReportCommand = this._mainView.addCommand({href: newReportURI, imageClass: "create-report-command", buttonText: this.messages.createReport, boldFirstWord: true, size: "small"});
			}
		
			if (this._canDeployQuery) {
				var myWidget = new com.ibm.team.reports.web.app.view.navigation.ReportsNavigator({controller: this.controller, shared: false, canDeployQuery: this._canDeployQuery, canManageFolder: this._canManageFolder, defaultQuery: this.controller.getDefaultQuery()});
				this._myReportsNavigator = myWidget;
				var navSection = this._mainView.addNavigationSection(myWidget.domNode, this.messages.myReports, true);
				if (this._canManageFolder) {
					this._myReportsNavigator.initializePulldown(navSection);
				}
				this.controller.initializeMyReportsNavigator(myWidget);
			}
			
			var sharedWidget = new com.ibm.team.reports.web.app.view.navigation.ReportsNavigator({controller: this.controller, shared: true, canDeployQuery: this._canDeployQuery, canManageFolder: this._canManageFolder, defaultQuery: this.controller.getDefaultQuery()});
			this._sharedReportsNavigator = sharedWidget;
			navSection = this._mainView.addNavigationSection(sharedWidget.domNode, this.messages.sharedReports, true);
			if (this._canManageFolder) {
				this._sharedReportsNavigator.initializePulldown(navSection);
			}
			this.controller.initializeSharedReportsNavigator(sharedWidget);

			var responseHandler = new ServiceResponseHandler(
				this,
				"_handleIsDWAdmin",
				"_remoteError");
			ReportsClient.getIsDWAdmin(responseHandler);
			
			// Include Archived checkbox
			var archivedWidget = new com.ibm.team.reports.web.app.view.navigation.IncludeArchived({controller: this.controller});
			//this._mainView._leftNav.appendChild(archivedWidget.domNode);
			this._mainView.addNavigationSection(archivedWidget.domNode, null, true);
		},
		
		_handleIsDWAdmin: function(result) {
			if (result == "true") {
				var widget = new com.ibm.team.reports.web.app.view.navigation.AdminNavigator({controller: this.controller});
				this._mainView.addNavigationSection(widget.domNode, this.messages.admin, true);
			}
		},

		// show methods to be called from ReportsController
		
		showQuery: function(args) {
			if (args.name) {
				PlatformUI.getWorkbench().setTitle(dojo.string.substitute(this.messages.reportWithName, [args.name]));
			} else {
				PlatformUI.getWorkbench().setTitle(this.messages.report);
			}
			var reportViewer = new com.ibm.team.reports.web.app.view.report.ReportViewer({queryUUID: args.queryUUID, controller: this.controller});
			this._mainView.setMainContent(reportViewer.domNode);
			if (this._myReportsNavigator != null && this._myReportsNavigator != undefined) {
				this._myReportsNavigator.setSelection({queryUUID: args.queryUUID});
			}
			if (this._sharedReportsNavigator != null && this._sharedReportsNavigator != undefined) {
				this._sharedReportsNavigator.setSelection({queryUUID: args.queryUUID});
			}
		}, 
		
		showReport: function(args) {
			PlatformUI.getWorkbench().setTitle(this.messages.report);
			var reportViewer = new com.ibm.team.reports.web.app.view.report.ReportViewer({isReport: true, reportUUID: args.reportUUID, parameterName: args.parameterName, parameterValue: args.parameterValue, controller: this.controller});
			this._mainView.setMainContent(reportViewer.domNode);
			if (this._myReportsNavigator != null && this._myReportsNavigator != undefined) {
				this._myReportsNavigator.setSelection(null);
			}
			if (this._sharedReportsNavigator != null && this._sharedReportsNavigator != undefined) {
				this._sharedReportsNavigator.setSelection(null);
			}
		}, 
		
		showWelcome: function() {
			var welcomePageWidget = new com.ibm.team.reports.web.app.view.report.ReportsWelcome({});
			this._mainView.setMainContent(welcomePageWidget.domNode);
		},
		
		newReport: function(args) {
			PlatformUI.getWorkbench().setTitle(this.messages.newReport);
			var newReportEditor = new com.ibm.team.reports.web.app.view.report.NewReportEditor({controller: this.controller});
			this._mainView.setMainContent(newReportEditor.domNode);
			if (this._myReportsNavigator != null && this._myReportsNavigator != undefined) {
				this._myReportsNavigator.setSelection(null);
			}
			if (this._sharedReportsNavigator != null && this._sharedReportsNavigator != undefined) {
				this._sharedReportsNavigator.setSelection(null);
			}
		},
		
		admin: function() {
			PlatformUI.getWorkbench().setTitle(this.messages.dataWarehouseAdministration);
			var adminViewer = new com.ibm.team.reports.web.app.view.admin.AdminViewer({controller: this.controller});
			this._mainView.setMainContent(adminViewer.domNode);
		},
	
		viewWorkItems: function(args) {
			WorkItemsNavigator.viewWorkItems(args);
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._appContainer, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(dojo.string.substitute(this.messages.errorWithMessage, [error.message]));
		}
	}
);

})();