/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.ContributorMemento");

dojo.require("com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.IMemento");

(function() {

var ContributorInfoRegistry = com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry;
var IMemento = com.ibm.team.dashboard.web.ui.IMemento;

/*
 * A memento that stores viewlet state data for a particular user, e.g. customized
 * appearance. It is stored in a ViewletContributorInfo helper of a
 * DashboardContributorInfo item.
 */
dojo.declare("com.ibm.team.dashboard.web.client.internal.ContributorMemento",
	IMemento, {
	
	constructor: function(/* string */ dashboardItemId, /* string */ viewletId) {
		this._dashboardItemId = dashboardItemId;
		this._viewletId = viewletId;
	},
	
	/*
	 * Returns the object or primitive stored under the given key for this
	 * memento, or undefined if there is none.
	 */
	/* object or primitive */ get: function(/* string */ key) {
		if (!this._memento) {
			var json = ContributorInfoRegistry.getMemento(this._dashboardItemId, this._viewletId);
			if (json) {
				this._memento = dojo.fromJson(json);
			}
			else {
				this._memento = new Object();
			}
		}
		return this._memento[key];
	},
	
	/*
	 * Stores the given value under the given key in this memento.
	 */
	set: function(/* string */ key, /* object or primative */ value) {
		if (!this._memento) {
			this._memento = new Object();
		}
		this._memento[key] = value;
		ContributorInfoRegistry.setMemento(this._dashboardItemId, this._viewletId, dojo.toJson(this._memento));
	}
});

})();