/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardWelcome");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardError");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.LeftNav");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletContainer");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.AllDashboards");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ActionRegistry = null; // bind to ActionRegistry later
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LeftNav = com.ibm.team.dashboard.web.ui.internal.LeftNav;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardContainer = com.ibm.team.dashboard.web.ui.internal.DashboardContainer;
var ViewletRegistry = com.ibm.team.dashboard.web.client.internal.ViewletRegistry;
var DashboardWelcome = com.ibm.team.dashboard.web.ui.internal.DashboardWelcome;
var DashboardError = com.ibm.team.dashboard.web.ui.internal.DashboardError;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var AllDashboards = com.ibm.team.dashboard.web.ui.internal.AllDashboards;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

/*
 * The widget containing the entire dashboard (left nav and content).
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardPage",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-DashboardPage\"></div>",
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		this._layout = new net.jazz.ajax.ui.SideNavigationLayout({});
		this.domNode.appendChild(this._layout.domNode);

		// create the left nav
		this._leftNav = new LeftNav({dashboardPage: this});
		
		// late bind to ActionRegistry to ensure that workbench is instantiated
		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("com.ibm.team.dashboard.viewDashboard", this, "viewDashboard");
		ActionRegistry.registerAction("com.ibm.team.dashboard.viewAllDashboards", this, "viewAllDashboards");
		ActionRegistry.registerAction("com.ibm.team.dashboard.viewDefaultDashboard", this, "viewDefaultDashboard");
	},
	init: function() {
		this.viewDashboard();
	},
	addListener: function(/* int */ type, /* function */ callback) {
		switch(type) {
			case Constants.Events.DashboardPage.DASHBOARD_CHANGED:
				if (!this._dashboardChangedListeners)
					this._dashboardChangedListeners = new Array();
				this._dashboardChangedListeners.push(callback);
				break;
			case Constants.Events.DashboardPage.ALL_DASHBOARDS_VIEWED:
				if (!this._allDashboardsListeners)
					this._allDashboardsListeners = new Array();
				this._allDashboardsListeners.push(callback);
				break;
		}
	},
	isVisible: function() {
		return this._isVisible;
	},
	navigateAwayEvent: function(event,workbenchNavigation) {
		if (event) {
			//if this function was called by an onbeforeunload event
			if (this._contentWidget && this._contentWidget._dashboardItem) {
				//return a message to show in the onBeforeUnload dialog
				return this._contentWidget.navigateAwayEvent(false, true);
			}
		} else {
			if (this.switchingTabs) {
				return true;
			}
			var navigateAwayParams = new Object();
			if (this._contentWidget && this._contentWidget instanceof DashboardContainer && this._contentWidget._dashboardItem) {
				navigateAwayParams = this._contentWidget.navigateAwayEvent(workbenchNavigation);
			} else {
				navigateAwayParams.doNavigate = true;
			}
			if (navigateAwayParams.doNavigate) {
				this._isVisible = false;
				if (navigateAwayParams.destroyWidget) {
					this._destroyContentWidget();
				}
				if (workbenchNavigation) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
					this.navigateAwayHandle = null;
					dojo.removeClass(document.body,"com-ibm-team-dashboard-body");
				}
			} else {
				this._leftNav.isExplicit = false;
			}
			return navigateAwayParams.doNavigate;	
		}
	},
	reload: function() {
		var contentWidget = this._contentWidget;
		if (contentWidget && contentWidget.getDashboardItem && contentWidget.getDashboardItem()) {
			contentWidget.reload();
		}
	},
	setLoading: function(isLoading, message) {
		if (isLoading) {
			
			PlatformUI.getWorkbench().showMessage(message);
		} else {
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},
	updateBrowserTitle: function() {
		var contentWidget = this._contentWidget;
		if (contentWidget && contentWidget.getDashboardItem && contentWidget.getDashboardItem()) {
			var dashboardTitle = contentWidget.getDashboardItem().getTitle();
			if (contentWidget.currentPage && contentWidget.currentPage.viewletPageHelper) {
				var tabTitle = contentWidget.currentPage.viewletPageHelper.getTitle();
				PlatformUI.getWorkbench().setTitle(dojo.string.substitute(this.messages.dashboardTitleWithTab, [dashboardTitle, tabTitle]));
			} else {
				PlatformUI.getWorkbench().setTitle(dojo.string.substitute(this.messages.dashboardTitle, [dashboardTitle]));
			}
		}
		else if (contentWidget && contentWidget instanceof AllDashboards) {
			if (contentWidget.tab && contentWidget.tab == 1)
				PlatformUI.getWorkbench().setTitle(this.messages.allUsersTitle);
			else
				PlatformUI.getWorkbench().setTitle(this.messages.allTeamAreasTitle);
		} else {
			PlatformUI.getWorkbench().setTitle(this.messages.dashboardsGenericTitle);
		}
	},
	viewDefaultDashboard: function(args) {
		args.showDefault = true;
		this.viewDashboard(args);
	},
	viewAllDashboards: function(args) {
		this._checkNavigateAwayHandle();
		// only perform left nav highlighting if all dashboards is not already open
		if (this._contentWidget instanceof AllDashboards) {
			this._contentWidget.select(args.tab, args.page);
		}
		else {
			this._leftNav.updateHighlighting({
				viewAllDashboards: true
			});
			if (this._contentWidget) {
				this._destroyContentWidget();
			}
			this._contentWidget = new AllDashboards({
				dashboardPage: this,
				tab: args.tab,
				page: args.page
			});
		}
		this.updateBrowserTitle();
		this._fireListeners(Constants.Events.DashboardPage.ALL_DASHBOARDS_VIEWED);
		this._layout.setMainContent(this._contentWidget.domNode);
		this._contentWidget.init();
	},
	viewDashboard: function(args) {
		this.updateBrowserTitle();
		this._isVisible = true;
		this.switchingTabs = false;
		this._checkNavigateAwayHandle();
		
		this._leftNav.updateExplicitHighlighting();
		
		// show the last dashboard when returning to dashboard page (if exists)
		if (args.showDefault && this._contentWidget && this._contentWidget._dashboardItem && !this._contentWidget.deleted) { // DashboardContainer already loaded (probably cached by workbench)
			dojo.publish("com.ibm.team.dashboard.web.onresize");
			if (this._contentWidget instanceof DashboardContainer) {
				this._contentWidget.setActive();
				// #48438: Dashboard actions/back button issues
				// include tab in url only if it's not the first one
				var tab;
				if (this._contentWidget.getCurrentPage() != this._contentWidget.viewletPages[0]) {
					tab = this._contentWidget.selectedTabId;
				}
				// allow click handlers to complete before switching
				setTimeout(dojo.hitch(this, this._updateURL, Util.getDashboardItemHref(this._contentWidget._dashboardItem, tab)), 100);
			}
			return;
		}
		// save args to compare future requests
		var previousArgs = this._latestArgs;
		this._latestArgs = args;

		// already open/requested?
		var tab = args.tab;
		if (!args.force && previousArgs) {
			// compare args with previous args
			var isSameDashboard = true;
			var props = ["id", "team", "user"];
			for (var i=0;i<props.length;i++) {
				var p = props[i];
				if (args[p] != previousArgs[p]) {
					isSameDashboard = false;
					break;
				}
			}
			if (isSameDashboard && this._contentWidget && this._contentWidget._dashboardItem && !this._contentWidget.deleted) {
				// only tab was different; switch tabs
				if (this._contentWidget instanceof DashboardContainer) {
					if (tab != previousArgs.tab) {
						this._contentWidget.getCurrentPage().setActive(false);
						this._contentWidget.metaNav.draggableTabs._selectTabById(tab);
						this.updateBrowserTitle();
					}
					this._contentWidget.setActive();
				}
				return;
			}
		}
		delete args.tab;
		delete args.action;
		
		if (!args.showDefault)
			this._leftNav.updateHighlighting(args);
		
		var serviceObj = {
			dashboardPage: this,
			args: args,
			tab: tab,
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "NOT_FOUND":
						// valid request but dashboard not yet created
						this.dashboardPage._doWelcome(this.args);
						break;
					case "ERROR":
						// invalid request or server error
						this.dashboardPage._createError.call(this.dashboardPage,responseDTO.msg);
						break;
					case "OK":
					default:
						/*
						 * Fill in the last request's details after we know what the default really is.
						 * Also update the highlighting now that we have the required info.
						 */
						if (this.args.showDefault) {
							if (this.dashboardPage._latestArgs.showDefault)
								this.dashboardPage._latestArgs = Util.getDashboardItemParams(responseDTO.data, this.tab);
							this.dashboardPage._leftNav.updateHighlighting(this.dashboardPage._latestArgs);
						}
						var contentWidget = this.dashboardPage._contentWidget;
						if(!responseDTO.data) {
							this.dashboardPage._doWelcome.call(this.dashboardPage,this.args);
						} else if (this.dashboardPage._isRequestedDashboardOpen.call(this.dashboardPage,responseDTO.data.getId())) {
							if (contentWidget.metaNav && contentWidget.metaNav.draggableTabs)
								contentWidget.metaNav.draggableTabs._selectTabById(this.tab);
						} else {
							if (contentWidget && contentWidget._dashboardItem) {
								this.dashboardPage._createContentWidget(this.tab);
							} 
							this.dashboardPage._initializeContentWidget.call(this.dashboardPage,responseDTO.data);
						}
						contentWidget = null;
						dojo.publish("com.ibm.team.dashboard.web.onresize");
						
						// #48438: Dashboard actions/back button issues
						if (this.args.showDefault) {
							this.dashboardPage._updateURL(Util.getDashboardItemHref(responseDTO.data));
						}
						break;
				}
			},
			failure: function(/* Error */ error) {
				if (error && error.status == 0)
					error.message = this.dashboardPage.messages.connectionFailure;
				this.dashboardPage._createError(this.dashboardPage.messages.errorRetrieving+"<br/><br/>" + error.message);
			}
		};
		this._createContentWidget(tab);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getDashboard(args, !ViewletRegistry.hasDefinitions(), srh);	
		
		setTimeout(dojo.hitch(this,this._preloadImages),0);
	},
	
	_deleteDashboard: function(scope, scopeItemId, isPublic, modified, force) {
		var serviceObj = {
			self: this,
			postDelete: function (descriptor) {
				// see if a dashboard is open
				if (this.self._contentWidget && this.self._contentWidget instanceof DashboardContainer) {
					var dashboardItem = this.self._contentWidget.getDashboardItem();
					// see if the deleted dashboard is open
					if (descriptor.itemId == dashboardItem.getId()) {
						this.self.setCurrentDeleted();
						if (scope == DashboardConstants.SCOPE_CONTRIBUTOR && !isPublic) {
							// deleting a private dashboard, go to default
							location.href = "#" + Constants.URL_PARAM_ACTION + "=" + Constants.URL_PARAM_ACTION_VIEW_PAGE + "&" + Constants.URL_PARAM_ACTION_VIEW_PAGE_ID + "=" + DashboardConstants.PAGE_ID;
						} else {
							// deleting a shared dashboard, return to it
							var url = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + (descriptor.urlParams ? ("&" + descriptor.urlParams) : "");
							if (location.href.substring(location.href.indexOf("#")) == url) {
								// already at the target href, call viewDashboard directly
								// can't be a private dashboard, so contributor scope means public
								if (scope == DashboardConstants.SCOPE_CONTRIBUTOR)
									var args = {user: descriptor.user};
								else if (scope == DashboardConstants.SCOPE_TEAM_AREA)
									var args = {team: decodeURIComponent(descriptor.team)};
								else
									var args = {};
								this.self.viewDashboard(args);
							} else {
								// not at target href, but viewing dashboard
								// must have tab param, so update url
								location.href = url;
							}
						}
					}
				}
			},
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "OK":
						this.self._leftNav._removeDashboardEntry(responseDTO.data.itemId);
						this.postDelete(responseDTO.data);
						break;
					case "STALE":
						if (responseDTO.msg) {
							// msg present means modified by someone else
							alert(dojo.string.substitute(this.self.messages.restoreFailure, [ responseDTO.msg ]));
						}
						else {
							// no msg means modified by current user
							alert(this.self.messages.saveFailure);
						}
						this.self._leftNav._removeDashboardEntry(responseDTO.data.itemId);
						this.postDelete(responseDTO.data);
						break;
					case "LOCKED":
						if (confirm(dojo.string.substitute(this.self.messages.restoreLockedOverride, [responseDTO.msg]))) {
							this.self._deleteDashboard(scope, scopeItemId, isPublic, null, true);
						}
						break;
					case "ERROR":
						alert(responseDTO.msg);
						break;
				}
				
			},
			failure: function(errorObj) {
				console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.deleteDashboard(scope, scopeItemId, isPublic, modified, force, srh);
	},
	
	setCurrentDeleted: function () {
		this._contentWidget.deleted = true;
	},
	_isRequestedDashboardOpen: function(args) {
		if (this._contentWidget instanceof DashboardContainer && this._contentWidget._dashboardItem) {
			var dashboardItem = this._contentWidget._dashboardItem;
			if (args.id) {
				return dashboardItem.getId() == args.id;
			} else if (args.user) {
				return dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && dashboardItem.isPublic() && dashboardItem.getUserId() == args.user;
			} else if (args.team) {
				return dashboardItem.getScope() == DashboardConstants.SCOPE_TEAM_AREA && unescape(dashboardItem.getTeamPath()) == args.team;
			} else {
				return dashboardItem.getScope() == DashboardConstants.SCOPE_PROJECT_AREA;
			}
		}
		return false;
	},
	_doWelcome: function(args) {
		var scope;
		var userId;
		var teamName;
		if (args && args.user) {
			scope = DashboardConstants.SCOPE_CONTRIBUTOR;
			userId = args.user;
		} else if (args && args.team) {
			scope = DashboardConstants.SCOPE_TEAM_AREA;
			teamName = args.team;
		} else {
			scope = DashboardConstants.SCOPE_PROJECT_AREA;
		}
		
		var self = this;
		var serviceObj = {
			success: function(/* WelcomeInfoDTO */ info) {
				self._createWelcome(scope, info, scope == DashboardConstants.SCOPE_CONTRIBUTOR);
			},
			failure: function(errorObj) {
				console.error("Dashboard.js remote error: " + errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getWelcomeInfo(scope, userId, teamName, srh);
	},
	_createContentWidget: function(/* string */ dashboardTabId) {
		if (this._contentWidget){
			this._destroyContentWidget();
		}
		if (!dashboardTabId)
			dashboardTabId = "tab1";
		this._contentWidget = new DashboardContainer({dashboardPage: this, selectedTabId: dashboardTabId});
		this._layout.setMainContent(this._contentWidget.domNode);
		this._contentWidget.init();
	},
	_initializeContentWidget: function(/* DashboardItem */ dashboardItem) {
		if (this._contentWidget && this._contentWidget.setContent) {
			dashboardItem._dashboardPage = this; //give dashboard item a reference back to the dashboardPage.
			this._contentWidget.setContent(dashboardItem);	
		}
			
		this._fireListeners(Constants.Events.DashboardPage.DASHBOARD_CHANGED);
	},
	_createWelcome: function(scope, /* WelcomeInfoDTO */ info, isPublic) {
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new DashboardWelcome({scope:scope, scopeItemId: info.scopeItemId, info: info, dashboardPage: this, _isPublic: isPublic});
		this._contentWidget.populate();
		this._layout.setMainContent(this._contentWidget.domNode);
		this._fireListeners(Constants.Events.DashboardPage.DASHBOARD_CHANGED);
	},
	_createError: function(/* string */ message) {
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new DashboardError({dashboardPage: this, message: message});
		this._contentWidget.populate();
		this._layout.setMainContent(this._contentWidget.domNode);
		this._fireListeners(Constants.Events.DashboardPage.DASHBOARD_CHANGED);
	},
	_fireListeners: function(/* int */ type) {
		switch(type) {
			case Constants.Events.DashboardPage.DASHBOARD_CHANGED:
				if (this._dashboardChangedListeners) {
					for (var i in this._dashboardChangedListeners)
						this._dashboardChangedListeners[i](this._contentWidget);
				}
				break;
			case Constants.Events.DashboardPage.ALL_DASHBOARDS_VIEWED:
				if (this._allDashboardsListeners) {
					for (var i in this._allDashboardsListeners)
						this._allDashboardsListeners[i]();
				}
				break;
		}
	},
	destroy: function() {
		if (this._leftNav && this._leftNav.destroy)
			this._leftNav.destroy();
		if (this._layout && this._leftNav.destroy)
			this._layout.destroy();
		if (this._contentWidget)
			this._contentWidget.destroy();
		this.inherited(arguments);
	},
	_destroyContentWidget: function() {
		if (!this._contentWidget) return;
		this._contentWidget.domNode.parentNode.removeChild(this._contentWidget.domNode);
		this._contentWidget.destroy();
		this._contentWidget = null;
	},
	_preloadImages: function() {
		if (this.pictures) return;
		
		this.pictures = new Array();
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-selected-left.gif")); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-selected-middle.gif")); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-selected-right.gif")); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-left.gif")); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-middle.gif")); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-right.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-down-arrow.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/tab-area-bg.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-left.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-right.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-mid.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/meta-nav-bg.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/viewlet-page-top-left.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/viewlet-page-top-right.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/close.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/edit-contents.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/down-arrow.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/minimize.gif"));  
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/expand.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/delete10.gif"));  
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/zoom-in.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/refresh.gif"));
		setTimeout(dojo.hitch(this,this._preloadSecondaryImages),4000);
	},
	_preloadSecondaryImages: function() {
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/close-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/down-arrow-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/minimize-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/expand-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/delete10-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/zoom-in-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/zoom-out.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/zoom-out-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/gray-corner-fill-TL.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/gray-corner-fill-TR.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/gray-corner-fill-BL.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/gray-corner-fill-BR.gif"));	
	},
	_checkNavigateAwayHandle: function() {
		if (!this.navigateAwayHandle) {
			this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			dojo.addClass(document.body,"com-ibm-team-dashboard-body");
		}
		if (!this.onBeforeUnloadHandle){
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);	
		}
	},
	_handleNavigate: function(urlObj) {
		var winLoc = new dojo._Url(window.location.href);
		var workbenchNavigation = false;
		if(urlObj.host && urlObj.scheme && !(urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) {
			// different domain - allow onBeforeUnload to prompt
			var navigateAway = true;
		} else if (urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path))) {
			// link within web UI
			if(urlObj.fragment.indexOf("com.ibm.team.dashboard") < 0) {
				// we're navigating to another area of the web UI
				workbenchNavigation = true;
			}
			if (this._stripTabParam(urlObj.fragment) == this._stripTabParam(winLoc.fragment)) {
				// the link is to the current url (not including the tab param)
				// ensure left nav highlighting is correct
				this._leftNav.updateExplicitHighlighting(/* clear explicit flag */true);
				var navigateAway = true;
			} else
				var navigateAway = this.navigateAwayEvent(false, workbenchNavigation);
			// same exact base URI - prompt for save/cancel
		} else {
			var navigateAway = true;
		}
		return !navigateAway;
	},
	_stripTabParam: function(string) {
		if (!string)
			return string;
		var index = string.indexOf("&tab=");
		if (index == -1)
			return string;
		var result = string.substring(0,index);
		var end = string.substring(index+1);
		index = end.indexOf("&");
		if (index == -1)
			return result;
		return result + end.substring(index);
	},
	/*
	 * Replaces the current URL with a new one, both for location and history.
	 */
	_updateURL: function(url) {
		// IE uses an iframe to track history
		if (dojo.isIE) {
			// see net.jazz.ajax.core.internal.IEUriMonitor
			var iframe = window["net.jazz.ajax.iframe"];
			if (iframe) {
				var href = iframe.location.href;
				var index = href.indexOf('?');
				if (index != -1) {
					// main frame will detect and update itself
					iframe.location.replace(href.substring(0, index) + "?" + url.substring(1));
					return;
				}
			}
		}
		location.replace(url);
	}
});

})();
