/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.LeftNav");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardWelcome");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.QuickSearch");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.FormatUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");

dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var QuickSearch = com.ibm.team.dashboard.web.ui.internal.QuickSearch;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var FormatUtil = com.ibm.team.dashboard.web.ui.internal.util.FormatUtil;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var MAX_RECENT = 5;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNav",
	dijit._Widget, {
		
	/*
	 * Creates the initial (empty) left nav content, and asynchronously
	 * populates it.
	 */
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		var leftNavSectionDiv = this.domNode;
		this._dashboardPage = args.dashboardPage;
		this._layout = args.dashboardPage._layout;
		
		this._layout._leftNav.style.display = "none";
		this._layout._showHideControl.style.display = "none";
		this._layout._contentArea.style.paddingLeft = "0";
		if (dojo.isIE == 6) {
			this._layout._contentArea.style.marginLeft = "0";
		}
		
		this._getLeftNavInfo();
		this._dashboardPage.addListener(Constants.Events.DashboardPage.DASHBOARD_CHANGED, dojo.hitch(this,this.hookDashboardListeners));
		this._layout._leftNav.appendChild(new QuickSearch({dashboardPage: this._dashboardPage}).domNode);
		return leftNavSectionDiv;
	},
	
	renameDashboard: function(/* DashboardItem */ dashboardItem) {
		if (dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && ! dashboardItem.isPublic()) {
			var node = this.findListNode(dashboardItem.getId());
			if (node) {
				this._doRename(node, dashboardItem.getTitle());
			}
			if (this._recentSection) {
				node = this._recentSection.getRow(dashboardItem.getId());
				if (node)
					this._doRename(node, dashboardItem.getTitle());
			}
			if (this._personalSection)
				this._personalSection.updateTitle(dashboardItem.getId(), dashboardItem.getTitle());
		}
	},
	
	_doRename: function(node, title) {
		node = node.lastChild.lastChild;
		while(node.firstChild)
			node.removeChild(node.firstChild);
		var span = document.createElement("span");
		span.innerHTML = FormatUtil.insertWordBreaks(HTMLUtility.escapeHTML(title),18);
		node.appendChild(span);
	},
	
	hookDashboardListeners: function(/* Widget */ contentWidget) {
		if (contentWidget instanceof com.ibm.team.dashboard.web.ui.internal.DashboardContainer)
			contentWidget.addListener(Constants.Events.DashboardContainer.DASHBOARD_RENAMED, dojo.hitch(this,this.renameDashboard))
	},
	
	updateExplicitHighlighting: function(/* boolean */ clearFlag) {
		if (this.isExplicit)
			this._doHighlighting(this.isExplicit);
		if (clearFlag)
			this.isExplicit = false;
	},
	
	updateHighlighting: function(args) {
		if (this.isExplicit) {
			this._doHighlighting(this.isExplicit);
			this.isExplicit = false;
			return;
		}
		// postpone highlighting until the left nav is fully loaded
		if (!this.isLoaded) {
			this._startHighlightTimer(args);
			return;
		}
		
		var rawArgs = args;
		args = this._translateArgs(args);
		
		if (this._updateHighlightingTimer) {
			clearTimeout(this._updateHighlightingTimer);
			delete this._updateHighlightingTimer;
		}
		var foundTarget = args.viewAllDashboards ? this._sharedSection.allDashboardsLink : null;
		if (!foundTarget) {
			switch(args.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (args.isPublic && args.userId == getAuthenticatedContributor().userId)
					foundTarget = this._sharedSection.profileLink;
				else if (args.isPublic)
					foundTarget = this.findListNode(args.userId);
				else
					foundTarget = this.findListNode(args.dashboardItemId);
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				foundTarget = this.findListNode(args.teamPath);
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				foundTarget = this._sharedSection.projectLink;
				break;
			}
		}
		// if we found a node, do the highlighting, otherwise clear it
		if (foundTarget) {
			this._doHighlighting(foundTarget);
		} else {
			// if we did not find a node, the recently viewed entry has not been created yet
			// try again later
			this._startHighlightTimer(rawArgs);
		}
	},
	
	_translateArgs: function(args) {
		if (args.viewAllDashboards)
			return args;
		var result = new Object();
		if (args.id) {
			result.scope = DashboardConstants.SCOPE_CONTRIBUTOR;
			result.dashboardItemId = args.id;
		} else if (args.user) {
			result.scope = DashboardConstants.SCOPE_CONTRIBUTOR;
			result.userId = args.user;
			result.isPublic = true;
		} else if (args.team) {
			result.scope = DashboardConstants.SCOPE_TEAM_AREA;
			result.teamPath = encodeURIComponent(args.team.replace(/\//g,"-"));
		} else {
			result.scope = DashboardConstants.SCOPE_PROJECT_AREA;
		}
		return result;
	},
	
	_explicitHighlight: function(/* Node */ node) {
		if (this._highlightedNode != node)
			this.isExplicit = node;
	},
	
	_startHighlightTimer: function(args) {
		this._updateHighlightingTimer = setTimeout(dojo.hitch(this,function(){
			this.updateHighlighting(args);
		}), 100);
	},
	
	_clearHighlighting: function() {
		if (this._highlightedNode) {
			dojo.removeClass(this._highlightedNode,Constants.LeftNav.SELECTED_DASHBOARD_CLASS);
			this._highlightedNode = null;
		}
	},
	
	_doHighlighting: function(newNode){
		if (this._highlightedNode != newNode) {
			if (this._highlightedNode)
				dojo.removeClass(this._highlightedNode,Constants.LeftNav.SELECTED_DASHBOARD_CLASS);
			dojo.addClass(newNode,Constants.LeftNav.SELECTED_DASHBOARD_CLASS);
			this._highlightedNode = newNode;
		}
	},
	
	/*
	 * Removes the given dashboard from the left nav (client side only).
	 */
	_removeDashboardEntry: function(dashboardItemId) {
		// if the deleted dashboard was selected, select the default dashboard
		var sectionRow = dojo.byId(dashboardItemId + Constants.LeftNav.LI_ID_SUFFIX);
		if (sectionRow) {
			sectionRow.parentNode.removeChild(sectionRow);
			this._personalSection.dashboardRemoved();
		}
		if (this._recentSection) {
			// try to remove from recently viewed based on dashboardId
			if (dashboardItemId)
				this._recentSection.removeExplicit(dashboardItemId);
				
		}
	},
	
	_getLeftNavInfo: function() {
		var serviceObj = {
			_leftNav: this,
			success: function(/* LeftNavDTO */ leftNavDTO) {
				var leftNav = this._leftNav;
				var perms = leftNavDTO.personalPermissions;
		
				if (perms.viewProject && (perms.viewTeam || perms.viewPersonal || perms.viewProfile)) {
					//show the leftnav
					leftNav._layout._leftNav.style.display = "";
					leftNav._layout._showHideControl.style.display = "";
					leftNav._layout._contentArea.style.paddingLeft = "";
					if (dojo.isIE == 6) {
						leftNav._layout._contentArea.style.marginLeft = "";
					}
				}
				 
				leftNav._navInfo = leftNavDTO;
				leftNav._populate();
			},
			failure: function(errorObj) {
				console.error(Constants.LeftNav.LEFT_NAV_INFO_ERROR + errorObj.message);
			}
		};
		var itemId = getAuthenticatedContributor().itemId;
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getLeftNavInfo(itemId, srh);
		return this.domNode;
	},
	
	_populate: function() {
		this._layout.addCommand({callback: dojo.hitch(this,this._handleNewPrivateDashboard), imageClass: "create-command", buttonText: this.messages.leftCreateButton, boldFirstWord: true, size: "small", disabled: !this._navInfo.personalPermissions.create});
		
		this._recentSection = new com.ibm.team.dashboard.web.ui.internal.LeftNavRecent({
			parent: this
		});
		this._recentSection.messages = this.messages;
		var recentNavSection = this._layout.addNavigationSection(this._recentSection, this.messages.leftRecentTitle, true);

		this._personalSection = new com.ibm.team.dashboard.web.ui.internal.LeftNavDashboards({
			personalDashboards: this._navInfo.personalDashboards,
			personalPermissions: this._navInfo.personalPermissions,
			parent: this
		});
		this._personalSection.messages = this.messages;
		var personalNavSection = this._layout.addNavigationSection(this._personalSection, this.messages.leftPersonalTitle, true);
		this._personalSection.populate(personalNavSection);
		
		this._sharedSection = new com.ibm.team.dashboard.web.ui.internal.LeftNavShared({
			processAreas: this._navInfo.projectAreas,
			personalPermissions: this._navInfo.personalPermissions,
			userID: this._navInfo.userID,
			parent: this
		});
		this._sharedSection.messages = this.messages;
		var processNavSection = this._layout.addNavigationSection(this._sharedSection, this.messages.leftSharedTitle, true);
		this._sharedSection.populate(processNavSection);
		this._recentSection.populate(recentNavSection);
		this.isLoaded = true;
	},
	
	_handleNewPrivateDashboard: function(event) {
		event.preventDefault();
		
		// check for unsaved dashboard
		if (this._dashboardPage.navigateAwayEvent(false)) {
			var self = this;
			var userItemId = getAuthenticatedContributor().itemId;
			var serviceObj = {
				success: function(/* ResponseDTO */ responseDTO) {
					switch(responseDTO.status) {
						case "OK":
							var dashboardItem = responseDTO.data;
							self._personalSection.doAddDashboard(dashboardItem.getId(), dashboardItem.getTitle());
							location.href= "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_ID + "=" + dashboardItem.getId();
							break;
						case "ERROR":
							alert(responseDTO.msg);
							break;
					}
				},
				failure: function(errorObj) {
					console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.createDashboard(DashboardConstants.SCOPE_CONTRIBUTOR, userItemId, false, srh);
		}
	},
	
	/* domNode */ findListNode: function(/* string */ id) {
		return dojo.byId(id + Constants.LeftNav.LI_ID_SUFFIX);
	},
	
	destroy: function() {
		if (this._personalSection)
			this._personalSection.destroy();
		if (this._sharedSection)
			this._sharedSection.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavRecent", dijit._Widget, {
	populate: function(/* NavSection */ navSection) {
		this.navSection = navSection;
		navSection.domNode.style.display = "none";
		this._count = 0;
		this._dashboards = {};
		this.parent._dashboardPage.addListener(Constants.Events.DashboardPage.DASHBOARD_CHANGED, dojo.hitch(this,this.dashboardOpened));
		this.parent._dashboardPage.addListener(Constants.Events.DashboardPage.ALL_DASHBOARDS_VIEWED, dojo.hitch(this,this.allDashboardsVisited));
		if (this.parent._dashboardPage._contentWidget) {
			var widget = this.parent._dashboardPage._contentWidget;
			if (widget instanceof com.ibm.team.dashboard.web.ui.internal.DashboardContainer)
				this.dashboardOpened(widget);
			else if (widget instanceof com.ibm.team.dashboard.web.ui.internal.AllDashboards)
				this.allDashboardsVisited();
		}
	},
	
	dashboardOpened: function(contentWidget) {
		if (!(contentWidget instanceof com.ibm.team.dashboard.web.ui.internal.DashboardContainer))
			return;
		var dashboardItem = contentWidget.getDashboardItem();
		if (!dashboardItem)
			return;
		// get the itemId
		if (dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !dashboardItem.isPublic())
			var itemId = dashboardItem.getId();
		else 
			var itemId = dashboardItem.getScopeItem().itemId ;
		if (dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && dashboardItem.isPublic())
			if (getAuthenticatedContributor().itemId == itemId)
				var existingSection = this.parent._sharedSection.profileLink;
		else if (dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR)
			var existingSection = this.parent.findListNode(itemId);
		else if (dashboardItem.getScope() == DashboardConstants.SCOPE_TEAM_AREA)
			var existingSection = this.parent.findListNode(dashboardItem.getTeamPath().replace(/\//g,"-"));
		else if (dashboardItem.getScope() == DashboardConstants.SCOPE_PROJECT_AREA)
			var existingSection = this.parent._sharedSection.projectSection;
		if (!this._dashboards[itemId]) {
			// get the title
			var title = dashboardItem.getTitle();
			// caluculate the iconUrl, title and href
			var href = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD;
			switch(dashboardItem.getScope()) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (dashboardItem.isPublic()) {
					var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/personal-dash-icon.gif");
					href += "&" + DashboardConstants.URL_PARAM_USER + "="  + dashboardItem.getUserId();
					if (getAuthenticatedContributor().itemId == itemId) 
						title = this.messages.leftProfileLink;
					else {
						title = dojo.string.substitute(this.messages.profileTitle, [dashboardItem.getScopeItem().name]);
						var nodeIdFragment = dashboardItem.getUserId();
					}
				}
				else {
					var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/private-dash-icon.gif");
					href += "&" + DashboardConstants.URL_PARAM_ID + "=" + itemId;
				}
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/team-dash-icon.gif");
				href += "&" + DashboardConstants.URL_PARAM_TEAM + "=" + dashboardItem.getTeamPath();
				title = dashboardItem.getScopeItem().name;
				var nodeIdFragment = dashboardItem.getTeamPath().replace(/\//g,"-");
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/project-dash-icon.gif");
				title = dashboardItem.getScopeItem().name;
				break;
			}
			
			this._count++;
			this.navSection.domNode.style.display = "";
			var sectionRow =  this.navSection.addSectionRow(FormatUtil.insertWordBreaks(HTMLUtility.escapeHTML(title),18), href, iconUrl, this._topSection);
			if (this._count > MAX_RECENT)
				this._removeOldest();
			if (!existingSection && nodeIdFragment) {
				sectionRow.id = nodeIdFragment + Constants.LeftNav.LI_ID_SUFFIX;
			}
			this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
			this._dashboards[itemId] = {
				lastVisit: new Date().getTime(),
				sectionRow: sectionRow
			};
			this._topSection = sectionRow;
		} else {
			this._dashboards[itemId].lastVisit = new Date().getTime();
		}
	},
	
	allDashboardsVisited: function() {
		if (!this._dashboards["allDashboards"]) {
			this._count++;
			this.navSection.domNode.style.display = "";
			var sectionRow =  this.navSection.addSectionRow(this.messages.allHeading, "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS, dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/all_dashboards_obj.gif"), this._topSection);
			if (this._count > MAX_RECENT)
				this._removeOldest();
			this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
			this._dashboards["allDashboards"] = {
				lastVisit: new Date().getTime(),
				sectionRow: sectionRow
			};
			this._topSection = sectionRow;
		} else {
			this._dashboards["allDashboards"].lastVisit = new Date().getTime();
		}
	},
	
	getRow: function(itemId) {
		if (this._dashboards && this._dashboards[itemId])
			return this._dashboards[itemId].sectionRow;
		return null;
	},
	
	removeExplicit: function(itemId) {
		var node = this.getRow(itemId);
		delete this._dashboards[itemId];
		if (node) {
			this._count--;
			if (this._topSection == node) {
				if (node.nextSibling)
					this._topSection = node.nextSibling;
				else
					this._topSection = null;
			}
			if (node.parentNode)
				node.parentNode.removeChild(node);
			if (!this._count)
				this.navSection.domNode.style.display = "none";
		}
	},
	
	_removeOldest: function() {
		var oldestVisit = new Date().getTime();
		var oldestDashboard;
		for (var i in this._dashboards) {
			if (this._dashboards[i].lastVisit < oldestVisit) {
				oldestDashboard = i;
				oldestVisit = this._dashboards[i].lastVisit;
			}
		}
		this._dashboards[oldestDashboard].sectionRow.parentNode.removeChild(this._dashboards[oldestDashboard].sectionRow);
		this._count--;
		delete this._dashboards[oldestDashboard];
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavDashboards", dijit._Widget, {
	populate: function(/* NavSection */navSection){
		this.navSection = navSection;
		navSection.domNode.style.display = "none";
		this._count = 0;
		this._titles = {};
		
		if (this.personalDashboards) {
			for (var i = 0; i < this.personalDashboards.length; i++) {
				this.doAddDashboard(this.personalDashboards[i].itemId, this.personalDashboards[i].title);
			}
		}
	},
	
	updateTitle: function(/* string */ itemId, /* string */title) {
		this._titles[itemId] = title;
	},
	
	doAddDashboard: function(/* string */ itemId, /* string */title) {
		this._count++;
		this._titles[itemId] = title;
		this.navSection.domNode.style.display = "";
		var href = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_ID + "=" + itemId;
		var sectionRow =  this.navSection.addSectionRow(FormatUtil.insertWordBreaks(HTMLUtility.escapeHTML(title),18), href, dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/private-dash-icon.gif"));
		sectionRow.id = itemId + Constants.LeftNav.LI_ID_SUFFIX;
		this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
		if (this.personalPermissions["delete"]) {
			var handleDelete = function(event) {
				dojo.stopEvent(event);
				this.deleteDashboard(itemId, this._titles[itemId]);
			};
			this.navSection.addArtifactCommand(sectionRow, this.messages.del, dojo.hitch(this, handleDelete), "delete-artifact-command", true);
		}
	},
	
	deleteDashboard: function(itemId, title) {
		if (confirm(dojo.string.substitute(this.messages.delConfirmPrivate, [title]))) {
			this.parent._dashboardPage._deleteDashboard(DashboardConstants.SCOPE_CONTRIBUTOR, itemId, false, null, false);
		}
	},
	
	dashboardRemoved: function() {
		this._count--;
		if (this._count == 0)
			this.navSection.domNode.style.display = "none";
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavShared",
	dijit._Widget, {
	
	populate: function(/* NavSection */ navSection) {
		if (this.personalPermissions) {
			var profileHref = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_USER + "=" + this.userID;
			this.profileLink = navSection.addSectionRow(this.messages.leftProfileLink, profileHref, dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/personal-dash-icon.gif"));
			this.connect(this.profileLink.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, this.profileLink));	
		}
		this.processAreas.sort(function(a,b) {
			return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1;
		});
		for (var i = 0; i < this.processAreas.length; i++) {
			var href = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD;
			var scope = (this.subPath || this.subPath == "") ? DashboardConstants.SCOPE_TEAM_AREA : DashboardConstants.SCOPE_PROJECT_AREA;
			if (scope == DashboardConstants.SCOPE_TEAM_AREA) {
				var currentPath = this.subPath + encodeURIComponent(this.processAreas[i].name);
				href += "&" + DashboardConstants.URL_PARAM_TEAM + "=" + currentPath;
				var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/team-dash-icon.gif");
			} else {
				var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/project-dash-icon.gif");
			}
			if (this.processAreas[i].children) {
				var subSectionWidget = new com.ibm.team.dashboard.web.ui.internal.LeftNavShared({
					processAreas: this.processAreas[i].children,
					subPath: currentPath ? currentPath + "/" : "",
					parent: this.parent
				});
				var subSection = navSection.addSubSection(subSectionWidget, FormatUtil.insertWordBreaks(HTMLUtility.escapeHTML(this.processAreas[i].name),18), href, iconUrl);
				subSectionWidget.depth = subSection.depth - 1;
				if (currentPath)
					subSection.navSectionEntry.id = currentPath.replace(/\//g,"-") + Constants.LeftNav.LI_ID_SUFFIX;
				else
					this.projectLink = subSection.navSectionEntry;
				subSectionWidget.populate(subSection);
				var sectionRow = subSection.navSectionEntry;
				this.connect(subSection._titleLink, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
			} else {
				var sectionRow = navSection.addSectionRow(FormatUtil.insertWordBreaks(HTMLUtility.escapeHTML(this.processAreas[i].name),18), href, iconUrl)
				if (currentPath)
					sectionRow.id = currentPath.replace(/\//g,"-") + Constants.LeftNav.LI_ID_SUFFIX;
				else
					this.projectLink = sectionRow;
				this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
			}
		}
		// the personalPermissions object only exists on the root widget
		// thus, test it before adding All Dashboards
		if (this.personalPermissions) {
			this.allDashboardsLink = navSection.addSectionRow(this.messages.allHeading, "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS, dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/images/all_dashboards_obj.gif"));
			this.connect(this.allDashboardsLink.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, this.allDashboardsLink));
		}
	}
});

})();
