/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.MetaNav");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier");

dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DraggableTabs = com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs;
var EventConstants = com.ibm.team.dashboard.web.ui.internal.Constants.Events.DashboardContainer;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ResizeNotifier = com.ibm.team.dashboard.web.ui.internal.util.resizeNotifier();

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var messages;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.MetaNav",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/MetaNav.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.refresh = messages.refresh;
		this.dashboardContainer = args.dashboardContainer;
	},
	postCreate: function() {
		var div = document.createElement("span");
		this._dashboardTitle.appendChild(div);
		this.title = this.dashboardContainer.createTitle(messages.loading,div,this.onTitleChange, null, null, "400px");
		dojo.addClass(this.title.domNode, "loading-title");
	},
	init: function(args) {		
		if (this.isInitialized) {
			return;
		}
		dojo.subscribe(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, this, "_showErrorMessage");
		dojo.subscribe(com.ibm.team.dashboard.web.ui.internal.Constants.hideServerError, this, "hideMessage");
		
		this.dashboardItem = this.dashboardContainer.getDashboardItem();
		this.viewletPageHelpers = args.viewletPageHelpers;
		
		var perms = this.dashboardItem.getPermissions();
		if (!this.dashboardContainer.isPrivateContributorDashboard() && (perms.viewTeam || perms.viewPersonal || perms.viewProfile))
			this._addBreadcrumbs(this.dashboardItem.getBreadcrumbs());
		else {
			this._breadcrumbs.style.display = "none";
			this._tabWrapper.style.marginTop = "3px";
		}
		
		this.title.value = args.title;
		this.title.domNode.innerHTML = HTMLUtility.escapeHTML(args.title);
		dojo.removeClass(this.title.domNode, "loading-title");
		//create new DraggableTabs container
		var div = document.createElement("div");
		this._tabs.appendChild(div);
		this.draggableTabs = new DraggableTabs(div,{parent: this, accept: ["tab"], selectedTabId: args.selectedTabId, dashboardContainer: this.dashboardContainer, dashboardItem: this.dashboardItem});
		
		for (var i=0; i<this.viewletPageHelpers.length; i++) {
			var viewletPageHelper = this.viewletPageHelpers[i];
			var id = viewletPageHelper.getId();
			this.draggableTabs.add(null,{title: viewletPageHelper.getTitle(), id: id, index: i, helper: viewletPageHelper, type: "tab"});
		}
		this._tabWrapper.style.display = "block";
		
		// show a notice if scope item is archived
		if (this.dashboardItem.isScopeArchived()) {
			var msg;
			var scope = this.dashboardItem.getScope();
			switch(scope) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					msg = messages.archivedContributor;
					break;
				case DashboardConstants.SCOPE_TEAM_AREA:
					msg = messages.archivedTeam;
					break;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					msg = messages.archivedProject;
					break;
			}
			this.showMessage({type: "info", message: msg});
		}
		this.draggableTabs.startup();
	},
	_showErrorMessage: function(errorObj) {
		console.warn(errorObj);
		this._errorState = true;
		if (!errorObj || errorObj.status == 0 || errorObj.status == 12002 || errorObj.status >= 12029)
			this.showMessage({message: messages.connectionError, actions: [{label: messages.reload, callback: dojo.hitch(this,this._cancelAreYouSure)}], type: "error"});
		else
			this.showMessage({message: messages.communicationError + " " + errorObj.message, actions: [{label: messages.reload, callback: dojo.hitch(this,this._cancelAreYouSure)}], type: "error"});
	},
	_addBreadcrumbs: function(/*Array*/ breadcrumbs) {
		for(var i = 0; i < breadcrumbs.length; i++) {
			var node = document.createElement("a");
			node.innerHTML = HTMLUtility.escapeHTML(breadcrumbs[i].label);
			node.href = breadcrumbs[i].href;
			var arrow = document.createTextNode(" > ");

			this._breadcrumbs.appendChild(node);
			this._breadcrumbs.appendChild(arrow);
		}
	},
	destroy: function() {
		if (this.title && this.title.destroy)
			this.title.destroy();
		if (this.draggableTabs) {
			this.draggableTabs.dashboardContainer = null;
			this.draggableTabs.destroy();	
		}
		this.inherited(arguments);
	},	
	setEditMode: function(editMode) {
		var beforeMode = this.editMode;
		this.editMode = editMode;
		if (editMode != beforeMode) {
			if (editMode == Constants.EDIT_AUTO_SAVE || editMode == Constants.EDIT_DEFERRED_SAVE) {
				this.draggableTabs.setEditMode(true);
				var perms = this.dashboardItem.getPermissions();
				if (perms.addTab) {
					this._createNewTabLink();	
				}
				if (this.titleClone) {
					this.titleClone.parentNode.replaceChild(this.title.domNode, this.titleClone);
				}
				
				this.addViewletButton = document.createElement("a");
				this.addViewletButton.id = "add-viewlet";
				this.addViewletButton.href = "#"
				this.addViewletButton.innerHTML = messages.addViewletText;
				this._options.appendChild(this.addViewletButton);
				if (editMode == Constants.EDIT_DEFERRED_SAVE) {
					this.dashboardItem.setDeferredMode(true);
					this.dashboardItem.setDirtyListener(this.setDirty, this);
					this.autoSave = false;
				}
				else {
					this.autoSave = true;
				}
				this.showSaveControls(true);
				this._setAutoSave();
			}
			else {
				this.draggableTabs.setEditMode(false);
				if (this.newTabLink) {
					this.newTabLink.parentNode.removeChild(this.newTabLink);
					this.newTabLink = null;
				}
				if (this.addViewletButton) {
					this.addViewletButton.parentNode.removeChild(this.addViewletButton);
					this.addViewletButton = null;
				}
				var div = document.createElement("div");
				div.innerHTML = HTMLUtility.escapeHTML(this.title.value);
				this.titleClone = div;
				this.title.domNode.parentNode.replaceChild(div, this.title.domNode);
				this.dashboardItem.removeDirtyListener();
				this.showSaveControls(false);
				if (typeof(this.beforeMode) != "undefined" && this.beforeMode != Constants.READ_ONLY) 
					this.setDirty(false);
			}
		}
	},
	showSaveControls: function (showControls){
		if(!this._editEvents) {
			this._editEvents = new Array();
		}
		if (showControls) {
			var autoSave = document.createElement("div");
			autoSave.className = "auto-save";
			var autoSaveInput = document.createElement("input");
			autoSaveInput.type = "checkbox";
			autoSaveInput.id = "autoSave-"+this.dashboardContainer._dashboardItem._id;
			
			var autoSaveLabel = document.createElement("label");
			autoSaveLabel.htmlFor = "autoSave-"+this.dashboardContainer._dashboardItem._id;
			autoSaveLabel.innerHTML = messages.autoSave;
			autoSave.appendChild(autoSaveInput);
			autoSave.appendChild(autoSaveLabel);
						
			var save = document.createElement("input");
			save.type = "button";
			save.value = messages.save;
			save.className = "saveButton";
			save.disabled = "disabled";
			if (typeof this.editMode != "undefined" && this.editMode == Constants.EDIT_AUTO_SAVE) 
				save.style.display = "none";
			
			this._headerLinks.appendChild(autoSave);
			this._headerLinks.appendChild(save);
			
			this._editEvents.push(this.connect(save,"onclick",this._saveEditingHandler));
			this._editEvents.push(this.connect(autoSaveInput,"onclick",this._setAutoSave));

			this.saveButton = save;
			this.autoSaveCheckbox = autoSave;
			this.autoSaveInput = autoSaveInput;
		} else {
			while(this._editEvents && this._editEvents.length > 0) {
				this.disconnect(this._editEvents.pop());	
			}
			if (this.saveButton && this.saveButton.parentNode) {
				this._headerLinks.removeChild(this.saveButton);				
			}
			if (this.autoSaveCheckbox && this.autoSaveCheckbox.parentNode) {
				this._headerLinks.removeChild(this.autoSaveCheckbox);
			}
			this.saveButton = null;
			this.autoSaveCheckbox = null;
			this.autoSaveInput = null;
		}
		if (!this.deleteButton && this.dashboardItem && this.dashboardItem.getPermissions()["delete"]) {
			var divider = document.createElement("span");
			this._headerLinks.insertBefore(divider,this._headerLinks.firstChild);
			
			var deleteButton = document.createElement("div");
			var deleteLink = document.createElement("a");
			if ((this.dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !this.dashboardItem.isPublic()) ||
					!this.dashboardItem.hasTemplate()) {
				deleteLink.className = "deleteIcon";
				deleteLink.title = messages.del;
			} else {
				deleteLink.className = "revertIcon";
				deleteLink.title = messages.restoreDefault;
			}
			deleteButton.appendChild(deleteLink)
			this.connect(deleteLink,"onclick",this._deleteDashboard);
			this.deleteButton = deleteButton;
			this._headerLinks.insertBefore(this.deleteButton,this._headerLinks.firstChild);
		}
	},
	_setAutoSave: function(event) {
		if((event && event.target.checked) || (!event && this.autoSave)) {
			this.saveButton.style.display = "none";
			this.setDirty(false);
			this.dashboardItem.save();
			this.dashboardItem.setDeferredMode(false);
			this.dashboardItem.removeDirtyListener();
			this.autoSave = true;
			this.autoSaveInput.checked = "checked";
		} else {
			this.saveButton.style.display = "";
			this.dashboardItem.setDeferredMode(true);
			this.dashboardItem.setDirtyListener(this.setDirty, this);
			this.autoSave = false;
		}
	},
	_saveEditingHandler: function(event){
		if (event) 
			dojo.stopEvent(event);
		this.dashboardItem.save();
		this.setDirty(false);
	},
	_cancelAreYouSure: function(event) {
		if (event) 
			dojo.stopEvent(event);
		if (this.dashboardItem && this.dashboardItem.isDirty() && !confirm(messages.confirm)) {
			return;	
		}
		if (this._errorState)
			this._cancelEditingHandler(event, false, true);
		else
			this._cancelEditingHandler(event, !(this.dashboardItem && this.dashboardItem.isDirty()));
	},
	_deleteDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var scope = this.dashboardItem.getScope();
		var isPublic = this.dashboardItem.isPublic();
		if(scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			if (isPublic) {
				var authContrib = getAuthenticatedContributor();
				if (this.dashboardItem.getScopeItem().itemId == authContrib.itemId) {
					var message = this.dashboardItem.hasTemplate() ? messages.restoreOwnPublicDashboard : messages.deleteOwnPublicDashboard;
					var scopeItemId = authContrib.itemId;
				}
				else {
					var name = this.dashboardItem.getScopeItem().name;
					var message = this.dashboardItem.hasTemplate() ? messages.restoreOtherPublicDashboard : messages.deleteOtherPublicDashboard;
					message = dojo.string.substitute(message, [ name ]);
					var scopeItemId = this.dashboardItem.getScopeItem().itemId;
				}
			} else {
				var message = dojo.string.substitute(messages.delConfirmPrivate, [this.dashboardItem.getTitle()]);
				var scopeItemId = this.dashboardItem.getId();
			}
		} else {
			var message = this.dashboardItem.hasTemplate() ? messages.restoreProcessDashboard : messages.deleteProcessDashboard;
			message = dojo.string.substitute(message, [this.dashboardItem.getScopeItem().name]);
			var scopeItemId = this.dashboardItem.getScopeItem().itemId;
		}
		if (!confirm(message))
			return;
		this.dashboardContainer.dashboardPage._deleteDashboard(scope, scopeItemId, isPublic, this.dashboardItem.getModified());
	},
	_cancelEditingHandler: function(event, refresh, error){
		if (event)
			dojo.stopEvent(event);
		if (error || this.dashboardItem.isDirty() || (event && event.ctrlKey)) {
			this._reload();
		} else if (refresh) {
			this._refresh();
		}
		// release lock; allow others to edit
		this.dashboardItem.unlock();
	},
	_refresh: function() {
		var serviceObj = {
			_metaNav: this,
			success: function(/* ResponseDTO */ responseDTO) {
				if (responseDTO.status == "OK" && responseDTO.msg) {
					// if timestamps match, refresh viewlet content
					if (this._metaNav.dashboardItem.getModified() == responseDTO.msg) {
						var page = this._metaNav.dashboardContainer.getCurrentPage();
						if (page) {
							page.refreshViewlets();
							return;
						}
					}
				}
				// full reload
				this._metaNav._reload();
			},
			failure: function(errorObj) {
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObj]);
			}
		};
		this.hideMessage();
		this._errorState = false;
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getModified(this.dashboardItem.getId(), srh)
	},
	// forces a reload of the current dashboard
	_reload: function() {
		var params = Util.getDashboardItemParams(this.dashboardItem, this.draggableTabs.selectedTabId);
		params.force = true;
		this.dashboardContainer.dashboardPage.viewDashboard(params);
	},
	setDirty: function(isDirty) {
		if (typeof isDirty == "undefined") isDirty = true;
		if (isDirty && !this.autoSave) {
			if (!this.dirtyIcon) {
				var span = document.createElement("span");
				span.innerHTML = "*";
				span.className = "dirtyStar";
				span.title = messages.dashboardModified;
				this._dashboardTitle.appendChild(span);
				this.dirtyIcon = span;	
			}
			this.saveButton.disabled = "";
		} else {
			if (this.dirtyIcon && this.dirtyIcon.parentNode) {
				this.dirtyIcon.parentNode.removeChild(this.dirtyIcon);
			}
			this.dirtyIcon = null;
			
			if (this.saveButton)
				this.saveButton.disabled = "disabled";
		}
	},
	/*
	 * Called when the user changes the dashboard title.
	 */
	onTitleChange: function(/* string */ newTitle) {
		this._dashboardItem.setTitle(newTitle);
		this._fireListeners(EventConstants.DASHBOARD_RENAMED);
		this.dashboardPage.updateBrowserTitle();
	},
	connectAddViewletButton: function(handler, context) {
		this.addViewletButtonHandler = dojo.hitch(context,handler);
		this.disconnect(this.addViewletButtonEvent);
		if (this.addViewletButton) {
			this.addViewletButtonEvent = this.connect(this.addViewletButton,"onclick",this.addViewletButtonHandler);
		}
	},
	showAddViewletButton: function() {
		if (this.addViewletButton) {
			this.addViewletButton.style.display = "";	
		}
	},
	hideAddViewletButton: function() {
		if (this.addViewletButton) {
			this.addViewletButton.style.display = "none";	
		}
	},
	_createNewTabLink: function() {
		//create new tab link
		if (!this.newTabLink) {
			var newTabLink = document.createElement("a");
			newTabLink.className = "add-new-tab";
			newTabLink.href = "#";
			newTabLink.title = messages.addNewTab;
			newTabLink.innerHTML = messages.addNewTab;
			this.newTabLinkEvent = this.connect(newTabLink,"onclick",this.draggableTabs.createNewTab);
			this._tabs.firstChild.appendChild(newTabLink);
			this.newTabLink = newTabLink;
		}
	},
	_removeNewTabLink: function(){
		if (this.newTabLink && this.newTabLink.parentNode) {
			this.disconnect(this.newTabLinkEvent);
			this.newTabLink.parentNode.removeChild(this.newTabLink);
			this.newTabLink = null;
		}
	},
	
	showMessage: function(args) {
		if (this._messageVisible) this._clearMessage();
		if (args) {
			if (args.type) {
				var compArr = ["error","warn","info"];
				for (var i = 0; i < compArr.length; i++) {
					if (compArr[i]==args.type) dojo.addClass(this._messageArea,"message-"+compArr[i]);
					else dojo.removeClass(this._messageArea,"message-"+compArr[i]);
				}
				if(args.type == "error") {
					dojo.addClass(this._messageArea,"message-error");
				} else if (args.type == "warn") {
					dojo.addClass(this._messageArea,"message-warn");					
				} else if (args.type == "info") {
					dojo.addClass(this._messageArea,"message-info");					
				}
			}
			if (args.message) {
				this._message.appendChild(document.createTextNode(args.message));
			}
			if (args.actions && args.actions instanceof Array) {
				if (!this._messageActionHandlers) this._messageActionHandlers = new Array();
				for (var i = 0; i < args.actions.length; i++) {
					var a = document.createElement("a");
					a.innerHTML = args.actions[i].label;
					if (args.actions[i].callback) {
						a.href="#";
						var callback = this._getCallback(args.actions[i].callback);
						this._messageActionHandlers.push(this.connect(a,"onclick",callback));
					} else if (args.actions[i].href) {
						a.href = args.actions[i].href;
					}
					this._action.appendChild(a);
				}
			}
			this._messageArea.style.display = "block";
			this._messageVisible = true;
			this._windowResizeEvent = ResizeNotifier.register({context: this, callback: this._onResizeHandler});
			setTimeout(dojo.hitch(this,this._onResizeHandler),200);
		}
	},
	_getCallback: function(callback) {
		return function(event) {
			event.preventDefault();
			callback();
		};
	},
	hideMessage: function() {
		this._clearMessage();
		this._messageArea.style.display = "";
		this._messageVisible = false;
		ResizeNotifier.unregister(this._windowResizeEvent);
	},
	_clearMessage: function() {
		while (this._messageActionHandlers && this._messageActionHandlers.length > 0) this.disconnect(this._messageActionHandlers.pop());
		while(this._action.firstChild)
			this._action.removeChild(this._action.firstChild);
		while(this._message.firstChild)
			this._message.removeChild(this._message.firstChild);
	},
	_onResizeHandler: function () {
		if (!this._messageNeedsRoom && this._messageArea.clientWidth < 160) {
			dojo.addClass(this._messageArea, "message-needs-room");
			this._messageNeedsRoom = true;
		} else if (this._messageNeedsRoom) {
			if (this.domNode.clientWidth - this._dashboardTitle.clientWidth - this._headerLinks.clientWidth - 43 >= 160) {
				dojo.removeClass(this._messageArea, "message-needs-room");
				this._messageNeedsRoom = null;
			}
		}
	}
});
})();