/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PagedTableMessages");

(function() {
	
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ResizeNotifier = com.ibm.team.dashboard.web.ui.internal.util.resizeNotifier();

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet",
	Viewlet, {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/PagedTableViewlet.html"),
	
	firstLinkImageSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","ui/internal/images/beginning.gif"),
	lastLinkImageSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","ui/internal/images/end.gif"),

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PagedTableMessages");
		this.next = messages.next;
		this.prev = messages.prev;
	},
	
	
		
	init: function() {
		var nodes = I18nUtil.addMessage(this._pageNumbers, messages.pageCount);
		this._pageText = nodes[0];
		this._currentPageFooter = nodes[1];
		this._totalPagesFooter = nodes[2];
		this._previousIsZoomed = false;
		this._currentPage = 1;
		this._updateCurrentPageFooter();
		this._loadingDiv.appendChild(document.createTextNode(this.getLoadingText()));
		// set src on images, so their size is correct in IE
		this._firstLinkImage.src = this.firstLinkImageSrc;
		this._lastLinkImage.src = this.lastLinkImageSrc;
		var contentClass = this.getContentClass();
		if (contentClass) {
			dojo.addClass(this._contentArea, contentClass);
			dojo.addClass(this._globalHeading, contentClass);
		}
		this.connect(this.domNode.parentNode,'onmouseover',this._showControls);
		this.connect(this.domNode.parentNode,'onmouseout',this._hideControls);
		this.connect(this._firstLink,'onmouseover',this._onMouseOverButton);
		this.connect(this._firstLink,'onmouseout',this._onMouseOutButton);
		this.connect(this._lastLink,'onmouseover',this._onMouseOverButton);
		this.connect(this._lastLink,'onmouseout',this._onMouseOutButton);
		if (dojo.isIE) {
			// if you click fast in IE, it omits clicks
			this.connect(this._prevLink,'ondblclick',this._handlePrevPage);
			this.connect(this._nextLink,'ondblclick',this._handleNextPage);
		}
		this._dashboardResizeEvent = ResizeNotifier.register({context: this, callback: this._onResize});
		this.onInit();
	},

	refresh: function() {
		this._entries = new Array();
		this._details = new Array();
		this._clearHeight();
		this._currentWidth = this._contentArea.offsetWidth;
		this.onRefresh();
		this.update();
	},
	
	update: function() {
		// track transitions between zoom and unzoom
		var isZoomed = this.isZoomed();
		if (isZoomed != this._previousIsZoomed) {
			if (isZoomed) {
				this._onZoom();
			}
			else {
				this._onUnzoom();
			}
			this._previousIsZoomed = isZoomed;
		}
		if (isZoomed)
			this._clearHeight();
		var itemsPerPage = this.getItemsPerPage();
		var resultCount = this.getResultCount();
		var startIndex = (this._currentPage - 1) * itemsPerPage;
		var entriesReady = true;
		var detailsReady = true;
		if (resultCount == -1) {
			entriesReady = false;
			detailsReady = !this.isZoomed();
		} else if (resultCount > 0 ){
			for (var i = startIndex; i < startIndex + itemsPerPage && i < resultCount; i++) {
				if (!this._entries[i]) 
					entriesReady = false;
				if (this.isZoomed() && !this._details[i]) 
					detailsReady = false;
				if (!entriesReady && (!this.isZoomed() || !detailsReady)) 
					break;
			}
		}
		var self = this;
		var serviceObject = {
			success: function(result) {
				if (result && result.entries && result.entries.length > 0) {
					for (var i = 0; i < result.entries.length; i++)
						self._entries[startIndex + i] = result.entries[i];
				}
				if (result && result.details && result.details.length > 0) {
					for (var i = 0; i < result.details.length; i++)
						self._details[startIndex + i] = result.details[i];
				}
				self._render();
				self._setLoading(false);
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
			},
			failure: function(error) {
				self._setLoading(false);
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		if (entriesReady && detailsReady) {
			this._render();
			this.getSite().doneRefresh();
		}
		else {
			this.getSite().setLoading(true);
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			this.fetch(srh, startIndex, itemsPerPage, !entriesReady, !detailsReady);
		}
	},
	
	/* TO BE OVERRIDEN BY SUBCLASSES IF NEEDED */
	
	// called when the viewlet is refreshed
	// can be used to invalidate caches etc.
	onRefresh: function() {
	},
	
	// called on viewlet initalization
	// can be used in place of init() function
	onInit: function() {
	},
	
	// called in the viewlet's destroy method
	// can be used to perform required cleanup
	onDestroy: function() {
	},
	
	// indicates the number of items to show on each page
	getItemsPerPage: function() {
		return 10;
	},
	
	// indicates the number of results in the current query
	// if the viewlet has no valid query or target, 0 should be returned
	// this will cause getNoResultsMessage to be called which allows the viewlet to be populated with a helpful message
	// if the viewlet has a valid query or target specified, but does not know the number of results yet, -1 should be returned
	getResultCount: function() {
		return 0;
	},
	
	// allows the viewlet to specify custom loading text
	getLoadingText: function() {
		return messages.loading;
	},
	
	// will be called when getResultCount returns 0
	// should return a domNode that will be placed in the content area of the viewlet
	getNoResultsMessage: function() {
		return document.createTextNode(messages.noResults);
	},
	
	// if subclasses wish to customize the styles of the viewlet
	// they can override this function and return a string
	// the result will be applied as the css class on the content area of the viewlet
	getContentClass: function() {
		return null;
	},
	
	// this will be called every time a page is display and will NOT be cached
	// if the entries or details for the page were not already available, then the heading will always
	//  be requested after the fetch for the required range completes
	// if the subclasses wishes to put a heading at the top of any given page then this function should
	//  return an array of n <td> domNodes that will be used to populate the table row
	// for no heading, return null
	/* Node[] */ getHeading: function(/* int */ startIndex, /* int */ count) {
		return null;
	},
	
	// this function will be called when more results are required
	// the startIndex and count parameters indicate which results should be retreived
	// the entries flag is used to indicate that the data for an unzoomed row is required
	// the details flag is used to indicate that the data for a zoomed row is required
	// 	(this flag will never be set on viewlets that are not zoomable)
	// once the results are available the provided ServiceResponseHandler's callback should be called
	//  with a single parameter that has an entries and/or a details property as required
	//  the array elements are viewlet-dependent data object that will be passed to the render
	//  methods for rendering
	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		srh.doErrorCallback(messages.fetchNotOverriden);
	},
	
	// this function will be called immediately before renderEntry and renderDetails
	// it can be used by subclasses to perform any calculations that depend on the entire
	// content of the current page before rendering indivitual entires
	preRender: function(/* object[] */ entries) {
	},
	
	// this function will be called when an entry needs to be rendered into DOM nodes
	// the entry data is one of the entry objects returned by the call to fetch, and is a
	//  viewlet-specific object containing the data for the entry
	// the details data is the details object returned by a fetch, also viewlet-specific, and will
	//  only be present if the zoomed flag is true. It is made available to provide additional
	//  details in the zoomed viewlet
	// the zoomed flag is for whether to render the zoomed (true) or unzoomed (false) entry
	// the return value must be an array of <td> DOM nodes that will be used to populate the table row
	//  this allows subclasses to specify custom css classes and column spans on the td elements
	/* Node[] */ renderEntry: function(/* object */ entryData, /* object */ detailsData, /* boolean */ zoomed) {
		console.error("renderEntry has not been overriden.");
	},
	
	// this function will be called when the details for an entry needs to be rendered into DOM nodes
	// the data is one of the details objects returned by the call to fetch, and is a viewlet-specific
	//   object containing the data for the details section
	// the return value must be an array of <td> DOM nodes that will be used to populate the table row
	//  this allows subclasses to specify custom css classes and column spans on the td elements
	/* Node[] */ renderDetails: function(/* object */ detailsData) {
		return null;
	},
	
	/* TO BE CALLED BY SUBCLASSES */
	
	// should be called when the total number of results changes
	// this will update the page count in the footer and show/hide it as required
	updateTotalPageFooter: function() {
		var pageCount = this._getPageCount()
		this._totalPagesFooter.innerHTML = pageCount;
		if (pageCount > 1)
			dojo.removeClass(this._footer, "hidden");
		else
			dojo.addClass(this._footer, "hidden");
	},
	
	// should be called when the viewlet needs to be reset to page 1
	// used when the query or target of the viewlet is changed or the number of items per page is changed
	// contains a call to updateTotalPageFooter, so subclasses need not call both in this case
	// a boolean parameter is used to indicate if the loading message should be displayed
	// the loading message is cleared the next time that:
	//  a) a call to fetch is performed and completed
	//  b) a no results message is displayed
	reset: function(/* boolean */ showLoading) {
		this._currentPage = 1;
		this.updateTotalPageFooter();
		this._updateCurrentPageFooter();
		this._clearHeight();
		if (showLoading)
			this._setLoading(true);
	},
	
	// can be called to set a heading that is independant of the current page
	// will always appear at the top of the viewlet except when the loading message is displayed
	setGlobalHeading: function(/* domNode */ node) {
		this.clearGlobalHeading();
		this._globalHeading.appendChild(node);
	},
	
	// used to clear the global heading
	clearGlobalHeading: function() {
		while (this._globalHeading.firstChild)
			this._globalHeading.removeChild(this._globalHeading.firstChild);
	},
	
	/* INTERNAL FUNCTIONS */
	
	_onZoom: function() {
		// don't auto-refresh while zoomed
		this.getSite().setAutoRefresh(false);
	},
	
	_onUnzoom: function() {
		// resume auto-refresh
		this.getSite().setAutoRefresh(true);
	},
	
	_render: function() {
		if (this.getResultCount() == 0) {
			this._renderNoResults();
			this._setLoading(false);
			return;
		}
		var itemsPerPage = this.getItemsPerPage();
		var startIndex = (this._currentPage - 1) * itemsPerPage;
		var endIndex = startIndex + itemsPerPage;
		var heading = this.getHeading((this._currentPage - 1) * this.getItemsPerPage(), this.getResultCount());
		if (!this._pageWidget) {
			this._pageWidget = new com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet.Page({viewlet: this});
			if (this._contentArea.firstChild)
				this._contentArea.replaceChild(this._pageWidget.domNode, this._contentArea.firstChild);
			else
				this._contentArea.appendChild(this._pageWidget.domNode);
			while(this._contentArea.firstChild.nextSibling)
				this._contentArea.removeChild(this._contentArea.firstChild.nextSibling);
			this._pageWidget.setContent(heading, this._entries.slice(startIndex, endIndex), this.isZoomed()? this._details.slice(startIndex, endIndex) : null, this.isZoomed());
		} else {
			if (!this.isZoomed() && !this._pageWidget.isZoomed() && !this._currentHeight && this._pageWidget.domNode.offsetHeight > 0) {
				this._currentHeight = this._pageWidget.domNode.offsetHeight;
				this._contentArea.style.height = this._currentHeight + "px";
			}
			this._pageWidget.setContent(heading, this._entries.slice(startIndex, endIndex), this.isZoomed()? this._details.slice(startIndex, endIndex) : null, this.isZoomed());
			if (!this.isZoomed() && this._currentHeight && this._pageWidget.domNode.offsetHeight > this._currentHeight) {
				this._currentHeight = this._pageWidget.domNode.offsetHeight;
				this._contentArea.style.height = this._currentHeight + "px";
			}
		}
	},
	
	_onResize: function() {
		if (this._currentWidth == this._contentArea.offsetWidth)
			return;
		this._currentWidth = this._contentArea.offsetWidth;
		this._clearHeight();
	},
	
	_clearHeight: function() {
		this._contentArea.style.height = "";
		delete this._currentHeight;
		if (this._shrinkTimer) {
			clearTimeout(this._shrinkTimer);
			delete this._shrinkTimer;
		}
	},
	
	_renderNoResults: function() {
		while(this._contentArea.firstChild)
			this._contentArea.removeChild(this._contentArea.firstChild);
		if (this._pageWidget) {
			this._pageWidget.destroy();
			delete this._pageWidget;
		}
		var messageDiv = document.createElement("div");
		messageDiv.className = "message";
		messageDiv.appendChild(this.getNoResultsMessage());
		this._contentArea.appendChild(messageDiv);
		this._clearHeight();
	},
	
	_doShrinkAnimation: function() {
		if (!this._pageWidget) {
			this._shrinkTimer = setTimeout(dojo.hitch(this,this._doShrinkAnimation), 100);
			return;
		}
		if (!this._currentHeight)
			return;
		delete this._shrinkTimer;
		if (!this.getSite().isActive()) {
			delete this._currentHeight;
			this._contentArea.style.height = "";
			return;
		}
		var endHeight = this._pageWidget.domNode.offsetHeight;
		var anim = dojo.animateProperty({node: this._contentArea, duration: 350, properties: {height: {start: this._currentHeight, end: endHeight, unit: "px"}}});
		delete this._currentHeight;
		this.connect(anim, "onEnd", function(){
			this._contentArea.style.height = "";
		});
		setTimeout(function(){anim.play();}, 0);
	},
	
	_handleNextPage: function(event) {
		dojo.stopEvent(event);
		if (this._currentPage == this._getPageCount()) {
			return;
		}
		
		this._currentPage++;
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_handlePrevPage: function(event) {
		dojo.stopEvent(event);
		if (this._currentPage == 1) {
			return;
		}
			
		this._currentPage--;
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_handleFirstPage: function(event) {
		dojo.stopEvent(event);
		this._first.blur();
		if (this._currentPage == 1) {
			return;
		}

		this._currentPage = 1;
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_handleLastPage: function(event) {
		dojo.stopEvent(event);
		this._last.blur();
		if (this._currentPage == this._getPageCount()) {
			return;
		}

		this._currentPage = this._getPageCount();
		this._updateCurrentPageFooter();
		this._showControls();
		this.update();
	},
	
	_setLoading: function(/* boolean */ visible) {
		if (visible) {
			dojo.removeClass(this._loadingDiv, "hidden");
			dojo.addClass(this._contentArea, "hidden");
			dojo.addClass(this._globalHeading, "hidden");
			dojo.addClass(this._footer, "hidden");
		} else {
			dojo.addClass(this._loadingDiv, "hidden");
			dojo.removeClass(this._contentArea, "hidden");
			dojo.removeClass(this._globalHeading, "hidden");
		}
	},
	
	_getPageCount: function() {
		return Math.ceil(this.getResultCount() / this.getItemsPerPage());
	},
	
	_showControls: function() {
		if (this._currentPage != 1) {
			this._firstLinkImage.src = this.firstLinkImageSrc;
			this._firstLinkDisabled = false;
			if (this._prev.firstChild != this._prevLink) {
				this._prev.replaceChild(this._prevLink, this._prev.firstChild);
			}
		} else {
			this._firstLinkImage.src = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","ui/internal/images/beginning-disabled.gif");
			this._firstLinkDisabled = true;
			if (this._prev.firstChild == this._prevLink) {
				var label = document.createElement("span");
				label.className = "disabled";
				label.innerHTML = messages.prev;
				this._prev.replaceChild(label, this._prev.firstChild);
			}
		}
		if (this._currentPage != this._getPageCount()) {
			this._lastLinkImage.src = this.lastLinkImageSrc;
			this._lastLinkDisabled = false;
			if (this._next.lastChild != this._nextLink) {
				this._next.replaceChild(this._nextLink, this._next.lastChild);
			}
		} else {
			this._lastLinkImage.src = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","ui/internal/images/end-disabled.gif");
			this._lastLinkDisabled = true;
			if (this._next.lastChild == this._nextLink) {
				var label = document.createElement("span");
				label.className = "disabled";
				label.innerHTML = messages.next;
				this._next.replaceChild(label, this._next.lastChild);
			}
		}
		dojo.removeClass(this._prev, "invisible");
		dojo.removeClass(this._next, "invisible");
		dojo.removeClass(this._first, "invisible");
		dojo.removeClass(this._last, "invisible");
		if (this._shrinkTimer) {
			clearTimeout(this._shrinkTimer);
			delete this._shrinkTimer;
		}
	},
	
	_hideControls: function(e) {
		if (dojo.isDescendant(e.relatedTarget, this.domNode.parentNode) == true)
			return;
		dojo.addClass(this._prev, "invisible");
		dojo.addClass(this._next, "invisible");
		dojo.addClass(this._first, "invisible");
		dojo.addClass(this._last, "invisible");
		if (!this.isZoomed())
			this._shrinkTimer = setTimeout(dojo.hitch(this,this._doShrinkAnimation), 2500);
	},
	
	_onMouseOverButton: function (e) {
		if (e.target == this._firstLink || e.target == this._firstLinkImage) {
			this.firstLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/images/beginning-hov.gif");
			if (!this._firstLinkDisabled)
				this._firstLinkImage.src = this.firstLinkImageSrc;
		} else {
			this.lastLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/images/end-hov.gif");
			if (!this._lastLinkDisabled)
				this._lastLinkImage.src = this.lastLinkImageSrc;
		}
	},
	
	_onMouseOutButton: function (e) {
		if (e.target == this._firstLink || e.target == this._firstLinkImage) {
			if (dojo.isDescendant(e.relatedTarget, this._firstLink) == true)
				return;
			this.firstLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/images/beginning.gif");
			if (!this._firstLinkDisabled)
				this._firstLinkImage.src = this.firstLinkImageSrc;
		} else {
			if (dojo.isDescendant(e.relatedTarget, this._lastLink) == true)
				return;
			this.lastLinkImageSrc = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/images/end.gif");
			if (!this._lastLinkDisabled)
				this._lastLinkImage.src = this.lastLinkImageSrc;
		}
	},
	
	_updateCurrentPageFooter: function() {
		this._currentPageFooter.innerHTML = this._currentPage;
	},
	
	destroy: function() {
		if (this._pageWidget)
			this._pageWidget.destroy();
		ResizeNotifier.unregister(this._dashboardResizeEvent);
		this.onDestroy();
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet.Page",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<table class=\"pageTable\" cellspacing=\"0\"><tbody dojoAttachPoint=\"tbody\"></tbody></table>",
	
	constructor: function(args) {
		this._viewlet = args.viewlet;
	},
	
	isZoomed: function() {
		return this._isZoomed;
	},
	
	setContent: function(/* Node[] */ heading, /* object[] */ entries, /* object[] */ details, /* boolean */ isZoomed) {
		// this._currentNode is used to replace existing table nodes
		this._isZoomed = isZoomed;
		this._currentNode = this.tbody.firstChild;
		while (this._currentNode && this._currentNode.nodeName == "#text")
			this._currentNode = this._currentNode.nextSibling;
		if (heading)
			this._doAppendReplaceNode(heading.contentRow);
		this._viewlet.preRender(entries);
		for (var i = 0; i < entries.length; i++) {
			this._doAppendReplaceNode(this._createRow(this._viewlet.renderEntry(entries[i], isZoomed ? details[i] : null, isZoomed)));
			if (details && details[i]) {
				this._doAppendReplaceNode(this._createRow(this._viewlet.renderDetails(details[i])));
			}
		}
		// remove any remaining old rows after populating the table
		while(this._currentNode) {
			var removeNode = this._currentNode;
			this._currentNode = this._currentNode.nextSibling;
			this.tbody.removeChild(removeNode);
		}
	},
	
	_createRow: function(/* Node[] */ columns) {
		var tr = document.createElement("tr");
		for (var i=0;i<columns.length;i++) {
			tr.appendChild(columns[i]);
		}
		return tr;
	},

	_doAppendReplaceNode: function(node) {
		if (this._currentNode) {
			var replaceNode = this._currentNode;
			this.tbody.replaceChild(node, replaceNode);
			this._currentNode = node.nextSibling;
		}
		else {
			this.tbody.appendChild(node);
		}
	},
	
	destroy: function() {
		while (this.tbody.firstChild)
			this.tbody.removeChild(this.tbody.firstChild);
	}

});

})();
