/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.PersonaViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PersonaMessages");

(function() {
	
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var MemberInfoClient = com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient;
var MemberInfo = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.PersonaViewlet",
	Viewlet, {
		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/PersonaViewlet.html"),

	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PersonaMessages");
		this.loading = messages.loading;
		this.notScoped = messages.notScoped;
	},

	init: function() {
		this._updateTitle();
		var nodes = I18nUtil.addMessage(this._editMessage, messages.notScopedEdit)
		this.connect(nodes[0],"onclick",this._handleToggleSettings);
		if (!this.getSite().isDashboardEditing())
			dojo.addClass(this._editMessage, "hidden");
	},
	
	refresh: function() {
		if (this._infoWidget) {
			dojo.removeClass(this._loading, "hidden");
			if (this._infoWidget.domNode && this.getScopeItem())
				this._viewletArea.style.height = this._infoWidget.domNode.offsetHeight + "px";
			this._infoWidget.destroy();
		}
		this._content.innerHTML = "";
		if (!this.getScopeItem()) {
			dojo.removeClass(this._message, "hidden");
			dojo.addClass(this._content, "hidden");
			dojo.addClass(this._loading, "hidden");
			this.getSite().doneRefresh();
			return;
		} else {
			dojo.addClass(this._message, "hidden");
		}
		this.getSite().setLoading(true);
		var serviceObj = {
			self: this,
			success: function(dtos) {
				this.self._infoWidget = new MemberInfo({memberInfoDto: dtos[0], linkTeams: this.self.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA), linkProjects: this.self.getSite().canViewDashboard(DashboardConstants.SCOPE_PROJECT_AREA)});
				this.self.getSite().setLoading(false);
				dojo.addClass(this.self._loading, "hidden");
				dojo.removeClass(this.self._content, "hidden");
				this.self._viewletArea.style.height = "";
				this.self._content.appendChild(this.self._infoWidget.domNode);
				this.self.getSite().doneRefresh();
			},
			failure: function(errorObj) {
				this.self.getSite().doneRefresh();
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObj]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getMemberInfo([this.getScopeItem().itemId], srh, this.getServiceTracker());
	},
	
	setDashboardEditing: function(editing) {
		if (editing) {
			dojo.removeClass(this._editMessage, "hidden");
		} else {
			dojo.addClass(this._editMessage, "hidden");
		}
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings.scopeItemId != oldSettings.scopeItemId) {
			this.getSite().refresh();
			var updateTitle = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle();
	},
	
	_updateTitle: function() {
		if (this.getScopeItem())
			this.getSite().setTitle(dojo.string.substitute(messages.titleScoped, [ this.getScopeItem().name ]));
		else
			this.getSite().setTitle(messages.title);
	}

});

})();