/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var PROJECT_ACTION_PREFIX = "com.ibm.team.process.editProjectArea&itemId="; //$NON-NLS-1$
var CONTRIBUTORS_IN_PROJECT_ACTION = "#action=com.ibm.team.process.usersInProjectArea&projectAreaItemId="; //$NON-NLS-1$

dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas", com.ibm.team.repository.web.ui.internal.admin.ItemList, { //$NON-NLS-1$
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		setText(this.heading, this.messages.ManageProjectAreas_1);
		setText(this.description, this.messages.ManageProjectAreas_2);
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.setHeaderRow([this.messages.ManageProjectAreas_3, this.messages.ManageProjectAreas_4]);
		this.headerRow.cells[0].style.width = "30%"; //$NON-NLS-1$
		this.addNewButton(PROJECT_ACTION_PREFIX + "new", this.messages.ManageProjectAreas_5); //$NON-NLS-1$
		this.refresh();
	},
	
	refresh: function () {
		this.clearState();
		var tr = document.createElement("tr"); //$NON-NLS-1$
		var td = document.createElement("td"); //$NON-NLS-1$
		td.colSpan = 2;
		td.appendChild(document.createTextNode(this.messages.ManageProjectAreas_6));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		var srh = new ServiceResponseHandler(this, "showProjects", "handleError"); //$NON-NLS-1$ //$NON-NLS-2$
		ProcessClient.getAllProjectAreas(srh, { hideArchivedProjects: true });
	},
	
	clearState: function () {
		dojox.data.dom.removeChildren(this.tbody);
		dojox.data.dom.removeChildren(this.messageDiv);
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = ""; //$NON-NLS-1$
	},
	
	showProjects: function(data) {
		dojox.data.dom.removeChildren(this.tbody);
		if (data != null) {
			var sortedData = data.sort(function (a,b) {
				return a.name > b.name ? 1 : a.name < b.name ? -1 : 0;
			});
			for (var i=0; i < sortedData.length; i++) {
				var actionHref = PROJECT_ACTION_PREFIX + sortedData[i].itemId;
				var descNode = document.createElement("div"); //$NON-NLS-1$
				descNode.appendChild(document.createTextNode(sortedData[i].summary));
				if (sortedData[i].description) {
					descNode.appendChild(document.createElement("br")); //$NON-NLS-1$
					descNode.appendChild(document.createElement("br")); //$NON-NLS-1$
					descNode.appendChild(document.createTextNode(sortedData[i].description));
				}
				var row = this.addRow(actionHref, [sortedData[i].name, descNode]);
				
				var archiveIcon = document.createElement("img"); //$NON-NLS-1$
				archiveIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				archiveIcon.title = this.messages.ManageProjectAreas_7;
				archiveIcon.className = "archiveIcon"; //$NON-NLS-1$
				archiveIcon.projectItemId = sortedData[i].itemId;
				this.connect(archiveIcon,"onmouseover", function (e) { //$NON-NLS-1$
					e.target.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive_hov.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				});
				this.connect(archiveIcon, "onmouseout", function (e) { //$NON-NLS-1$
					e.target.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				});
				this.connect(archiveIcon, "onclick", function (e) { //$NON-NLS-1$
					dojo.stopEvent(e);
					if (confirm(this.messages.ManageProjectAreas_8)) {
						this.archiveProject(e.target.projectItemId);
					}
				});
				
				row.cells[0].insertBefore(archiveIcon, row.cells[0].firstChild);
				
				var contributorsInProject = this._createContributorImage(sortedData[i], "users-in-project", this.messages.ManageProjectAreas_13, false); //$NON-NLS-1$
				row.cells[0].insertBefore(contributorsInProject, row.cells[0].firstChild);
				
				var contributorsNotInProject = this._createContributorImage(sortedData[i], "users-not-in-project", this.messages.ManageProjectAreas_14, true); //$NON-NLS-1$
				row.cells[0].insertBefore(contributorsNotInProject, row.cells[0].firstChild);
				
				dojo.toggleClass(row.parentNode, "projectTable", true); //$NON-NLS-1$
			}
		}
		else {
			this.messageDiv.appendChild(document.createTextNode(this.messages.ManageProjectAreas_9));
		}
	},
	
	_createContributorImage : function(projectArea, imageName, title, notInProject) {
		
		var imagePath = "ui/internal/admin/projects/images/" + imageName + ".gif"; //$NON-NLS-1$ //$NON-NLS-2$
		
		var image = document.createElement("img"); //$NON-NLS-1$
		image.src = dojo.moduleUrl("com.ibm.team.process.web", imagePath); //$NON-NLS-1$ 
		image.title = title;
		image.className = "usersIcon"; //$NON-NLS-1$
		image.projectItemId = projectArea.itemId;
		image.actionHref =  CONTRIBUTORS_IN_PROJECT_ACTION + projectArea.itemId + "&projectAreaName=" + projectArea.name + "&notInProject=" + notInProject; //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(image,"onmouseover", function (e) { //$NON-NLS-1$
			e.target.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/" + imageName + "-hov.gif"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		});
		this.connect(image, "onmouseout", function (e) { //$NON-NLS-1$
			e.target.src = dojo.moduleUrl("com.ibm.team.process.web", imagePath); //$NON-NLS-1$
		});
		this.connect(image, "onclick", function (e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			window.location.hash = e.target.actionHref;
		});
		
		return image;
	},
	
	archiveProject: function (itemId) {
		var srh = new ServiceResponseHandler(this, "archiveCallback", "archiveErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
		this.displayPendingMessage(this.messages.ManageProjectAreas_10);
		ProcessClient.postArchiveProjectArea({ uuid: itemId }, srh);
	},
	
	archiveCallback: function () {
		this.refresh();
		this.displaySuccessMessage(this.messages.ManageProjectAreas_11);
	},
	
	archiveErrorCallback: function (err) {
		this.displayErrorMessage(dojo.string.substitute(this.messages.ManageProjectAreas_12, [ err.message ]));
	}
});

})();