/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TeamParentSelector"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.internal.Dialog"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TeamParentSelector", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		
	templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TeamParentSelector.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.hierarchy = args;
	},
	
	postCreate: function () {
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.TeamParentSelector_1,
			isClosable: true
		});
		this.populateObj();
	},
	
	close: function () {
		this.dialog.close();
	},
	
	execute: function () {},
	
	populateObj: function () {
		dojox.data.dom.removeChildren(this.selectorBody);
		this.allRadios = new Array();
		this.finishButton.disabled = true;
		if (this.hierarchy != null) {
			for (var i=0; i < this.hierarchy.length; i++) {
				this.addElement(this.hierarchy[i]);
			}
		}
		else {
			this.selectorBody.appendChild(document.createTextNode(this.messages.TeamParentSelector_2));
		}
	},
	
	addElement: function (elem, parentNode, parentRadio) {
		var div = document.createElement("div"); //$NON-NLS-1$
		div.style.marginTop = "0.4em"; //$NON-NLS-1$
		var radio;
		var radioName = "parentElem"; //$NON-NLS-1$
		if (dojo.isIE) {
			// Work around the IE bug that causes radio buttons to be unselectable if the type is defined after the element is created
			radio = document.createElement("<input type=\"radio\" name=\"" + radioName + "\" >"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		else {
			radio = document.createElement("input"); //$NON-NLS-1$
			radio.type = "radio"; //$NON-NLS-1$
			radio.name = radioName;
		}
		radio.value = elem.itemId;
		radio.style.marginRight = "0.5em"; //$NON-NLS-1$
		radio.style.verticalAlign = "middle"; //$NON-NLS-1$
		radio.id = radio.name + ":" + radio.value; //$NON-NLS-1$
		radio.elemName = elem.name;
		if (parentRadio != null) {
			radio.parentRadio = parentRadio;
		}
		this.allRadios.push(radio);
		div.radio = radio;
		div.appendChild(radio);
		var labelNode = document.createElement("label"); //$NON-NLS-1$
		labelNode.htmlFor = radio.name + ":" + radio.value; //$NON-NLS-1$
		labelNode.appendChild(document.createTextNode(elem.name));
		div.appendChild(labelNode);
		if (elem.children != null && elem.children.length > 0) {
			var childNodeDiv = document.createElement("div"); //$NON-NLS-1$
			childNodeDiv.style.marginTop = "0.4em"; //$NON-NLS-1$
			childNodeDiv.style.paddingLeft = "1.5em"; //$NON-NLS-1$
			div.childNodeDiv = childNodeDiv;
			for (var i=0; i < elem.children.length; i++) {
				this.addElement(elem.children[i], childNodeDiv, radio);
			}
			div.appendChild(childNodeDiv);
		}
		if (parentNode == null) {
			this.selectorBody.appendChild(div);
		}
		else {
			parentNode.appendChild(div);
		}
		this.connect(radio,"onclick",function (e) { //$NON-NLS-1$
			this.finishButton.disabled = false;
		});
	},
	
	getChosenArea: function () {
		var chosenArray = new Array();
		for (var i=0; i < this.allRadios.length; i++) {
			if (this.allRadios[i].checked) {
				var curObj = this.allRadios[i];
				while (curObj != null) {
					chosenArray.unshift({
						name: curObj.elemName,
						itemId: curObj.value
					});
					curObj = curObj.parentRadio;
				}
			}
		}
		return chosenArray;
	}
});

})();