/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.common.ProgressLabelProvider"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var bind= dojo.string.substitute;

dojo.declare("com.ibm.team.apt.web.ui.internal.common.ProgressLabelProvider", null, { //$NON-NLS-1$

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_progressInfo: null,
	
	constructor: function(progressInfo) {
		this._progressInfo= progressInfo;
		this._messages = dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getHoursTextNode: function() {
		var result= document.createElement("span"); //$NON-NLS-1$
		result.appendChild(this._textNode(this._messages.PROGRESS_HOUR_NODE_PREFIX, "type")); //$NON-NLS-1$
		result.appendChild(this._labelNode(this.getWorkHoursDoneLabel()));
		result.appendChild(this._textNode("/", "separator")); //$NON-NLS-1$ //$NON-NLS-2$
		result.appendChild(this._labelNode(this.getWorkHoursLabel()));
		if (this._progressInfo.isRealTimeSpecified() && !this._progressInfo.isAllWorkDone()) {
			result.appendChild(this._textNode("|", "separator")); //$NON-NLS-1$ //$NON-NLS-2$
			result.appendChild(this._labelNode(this.getDeltaLabel()));
		}
		result.appendChild(this._textNode(this._messages.PROGRESS_HOUR_NODE_UNIT, "separator")); //$NON-NLS-1$

		return result;
	},

	getEstimateTextNode: function() {
		var result= document.createElement("span"); //$NON-NLS-1$
		result.appendChild(this._textNode(this._messages.PROGRESS_QOP_NODE_PREFIX, "type")); //$NON-NLS-1$
		result.appendChild(this._labelNode(this.getQualityOfPlanningLabel()));

		return result;
	},

	getDeltaLabel: function() {
		var result= this._createDefaultLabel("-", this._messages.PROGRESS_DELTA_LABEL); //$NON-NLS-1$

		if (this._hasEstimatedWork()) {
			var workHoursDoneDelta= this._progressInfo.getWorkHoursDoneDelta();
			
			result.value= workHoursDoneDelta + ""; //$NON-NLS-1$
			if (workHoursDoneDelta > 0) 
				result.value= "+" + result.value; //$NON-NLS-1$
			result.className= this._getWorkHoursDoneClass();
		}
		return result;
	},

	getWorkHoursLabel: function() {
		var result= this._createDefaultLabel("0", this._messages.PROGRESS_WORKHOURS_LABEL); //$NON-NLS-1$
		result.value= this._progressInfo.getWorkHours() + ""; //$NON-NLS-1$
		result.className= null;
		return result;
	},

	getWorkHoursDoneLabel: function() {
		var result= this._createDefaultLabel("0", this._messages.PROGRESS_WORKHOURSDONE_LABEL); //$NON-NLS-1$
		result.value= this._progressInfo.getWorkHoursDone() + ""; //$NON-NLS-1$
		result.className= null;
		return result;
	},

	getQualityOfPlanningLabel: function() {
		var result= this._createDefaultLabel("-", this._messages.PROGRESS_QOP_LABEL); //$NON-NLS-1$

		if (this._hasWork() && !this._progressInfo.isAllWorkDone()) {
			result.value= Math.round(this._progressInfo.getQualityOfPlanning()) + "%"; //$NON-NLS-1$
			result.className= null;
		}
		
		return result;
	},

	getTooltipContent: function() {
		var lines= [];
		lines.push(this._messages.PROGRESS_TOOLTIP_TITLE.bold());
		lines.push(bind(this._messages.PROGRESS_TOOLTIP_DONE, [this._progressInfo.getWorkHoursDone(), this._progressInfo.getWorkHours()]));

		if (this._progressInfo.isRealTimeSpecified() && !this._progressInfo.isAllWorkDone()) {
			var deltaLabel= null;
			switch (this._getWorkHoursDoneClass()) {
				case "ahead": //$NON-NLS-1$
					deltaLabel= bind(this._messages.PROGRESS_TOOLTIP_AHEAD, [Math.abs(this._progressInfo.getWorkHoursDoneDelta())]);
					break; 
				case "behind": //$NON-NLS-1$ 
					deltaLabel= bind(this._messages.PROGRESS_TOOLTIP_BEHIND, [Math.abs(this._progressInfo.getWorkHoursDoneDelta())]);
					break; 
				case "ontime": //$NON-NLS-1$ 
					deltaLabel= this._messages.PROGRESS_TOOLTIP_ONTIME;
					break; 
			}
			lines.push(bind(this._messages.PROGRESS_TOOLTIP_EXPECTED, [this._progressInfo.getWorkHoursDoneExpected(), deltaLabel]));
		}
		if (!this._progressInfo.isAllWorkDone()) {
			lines.push(""); //$NON-NLS-1$
			lines.push(bind(this._messages.PROGRESS_TOOLTIP_QOP, [Math.round(this._progressInfo.getQualityOfPlanning())]));
			lines.push(bind(this._messages.PROGRESS_TOOLTIP_CLOSED, [this._progressInfo.getCloseCount(), this._progressInfo.getCount(), Math.round(this._progressInfo.getClosedRatio())]));
		} else {
			lines.push(this._messages.PROGRESS_TOOLTIP_CLOSED_ALL);
		}

		return lines.join("<br />"); //$NON-NLS-1$
	},

	getMessage: function() {
//		if (this._progressInfo.isRealTimeSpecified() && this._progressInfo.getRealTimeLeft() == 0) {
//			return "No Work Time Left";
//		} 
		if (!this._hasWork()) {
			return this._messages.PROGRESS_BAR_NOWORK;
		} else if (!this._hasEstimatedWork() && this._progressInfo.getOpenCount() > 0) {
			return this._messages.PROGRESS_BAR_NOTESTIMATED;
		}

		return null;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_hasWork: function() {
		return this._progressInfo.getCount() > 0;
	},

	_hasEstimatedWork: function() {
		return this._hasWork() && this._progressInfo.getEstimatedCount() > 0;
	},

	_createDefaultLabel: function(noValue, tooltip) {
		return {
			value: noValue || this._messages.PROGRESS_NOVALUE,
			className: "na", //$NON-NLS-1$
			tooltip: tooltip
		};
	},

	_getWorkHoursDoneClass: function() {
		var workHoursProgress= this._progressInfo.getWorkHourProgress();
		var realTimeProgress= this._progressInfo.getRealTimeProgress();
		
		var result = "ontime"; //$NON-NLS-1$
		if (Math.abs(realTimeProgress - workHoursProgress) > 0.02) {
			result= realTimeProgress > workHoursProgress ? "behind" : "ahead"; //$NON-NLS-1$ //$NON-NLS-2$
		}
		return result;
	},
	
	_labelNode: function(label) {
		var result= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(result, "value"); //$NON-NLS-1$
		dojo.addClass(result, label.className || ""); //$NON-NLS-1$
		result.innerHTML= label.value;
		result.setAttribute("title", label.tooltip || ""); //$NON-NLS-1$ //$NON-NLS-2$ 
		return result;
	},

	_textNode: function(text, className) {
		var result= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(result, className || ""); //$NON-NLS-1$
		result.appendChild(document.createTextNode(text));
		return result;
	},


	__sentinel: null // terminates this class definition
});

})();
