/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ParameterNameChooserDialog");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("dojo.string");
dojo.require("dijit.Dialog");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StatisticsMessages");

/*
 * Aliases 
 */


(function() {

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ParameterNameChooserDialog",
	dijit.Dialog, {
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ParameterNameChooserDialog
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StatisticsMessages");
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited("postCreate", arguments);
		
		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->

		this._contentContainer = null;
		this._buttonContainer = null;
		this._listBox = null;
		this._okButton = null;
		this._cancelButton = null;
		
		this._parameters = null;
		
		this._callBackDialogExit = null;
		
		this._listSize = this._static._DEFAULT_LIST_SIZE;
		
		this.reset();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	reset: function() {
		this._selectedParameterName = null;
		this._selectedParameterValue = null;		
		this._doIt = false;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	show: function() {
		// Validate parameters
		if (this._parameters == null) {
			// Parameters need to be set before show
			return;
		}
		// Create the UI
		this._createUI();
		// Call super
		this.inherited("show", arguments);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	hide: function() {
		// Call super
		this.inherited("hide", arguments);
		// Fire dialog exit event
		this._fireEventDialogExit();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_fireEventDialogExit: function() {
		// Ensure a call back was defined
		if (this._callBackDialogExit == null) {
			return;
		}
		// Create the event
		var event = new Object();
		event.queryItemID = this._parameters.queryItemID;
		event.parameterID = this._parameters.parameterID;
		event.parameterName = this._selectedParameterName;
		event.parameterValue = this._selectedParameterValue;
		event.doIt = this._doIt;
	 	
		// Execute the defined call back
		this._callBackDialogExit.handleEventDialogExit(event);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 */	
	isOpen: function() {
		return this.open;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object args
 	 * 			. Object self
 	 * 				. Function handleEventDialogExit(event)
 	 * @return boolean
 	 */	
	setCallBackDialogExit: function(args) {
		// Set the listener
		this._callBackDialogExit = args.self;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object parameters
 	 * 			. Array parameterNames - String values
 	 * 			. Array parameterValues - Number values
 	 * 			. String queryItemID
 	 * 			. String parameterID
 	 * @return void
 	 */	
	setParameters: function(parameters) {
		this._parameters = parameters;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Number listSize
 	 * @return void
 	 */	
	setListSize: function(listSize) {
		if (listSize < 1) {
			this._listSize = this._static._DEFAULT_LIST_SIZE;
		} else if (listSize > this._static._MAX_LIST_SIZE) {
			this._listSize = this._static._MAX_LIST_SIZE;
		} else {
			this._listSize = listSize;
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Object parameters
 	 * 			. Array parameterNames - String values
 	 * 			. Array parameterValues - Number values
 	 * 			. String queryItemID
 	 * 			. String parameterID
 	 */	
	getParameters: function(parameters) {
		return this._parameters;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_createUI: function() {
		// Content container
		this._contentContainer = this._createUIContentContainer(this.domNode);
		// Scrolled list box
		this._listBox = this._createUIListBox(this._contentContainer);
		// Button container
		this._buttonContainer = this._createUIButtonContainer(this.domNode);
		// OK button
		this._okButton = this._createUIOKButton(this._buttonContainer);
		// Cancel button
		this._cancelButton = this._createUICancelButton(this._buttonContainer);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return Element
 	 */	
	_createUIContentContainer: function(parent) {
		// Content container
		var contentContainer = document.createElement("div");
		parent.appendChild(contentContainer);
		return contentContainer;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return Element
 	 */	
	_createUIButtonContainer: function(parent) {
		// Content container
		var buttonContainer = document.createElement("div");
		parent.appendChild(buttonContainer);
		return buttonContainer;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return Element
 	 */	
	_createUIListBox: function(parent) {
		// Scrolled list box
		var listBox = document.createElement("select");
		listBox.setAttribute("size", this._listSize);
		listBox.style.width = "230px";
		listBox.style.overflow = "auto";
		listBox.style.margin = "0px 10px 10px 10px";
		
		this.connect(listBox, "ondblclick", this._handleEventOKButtonClicked);
		
		// Get parameter names
		var parameterNames = this._parameters.parameterNames;
		// Get parameter values
		var parameterValues = this._parameters.parameterValues;
		// Create options for all parameters
		for (var i = 0; i < parameterNames.length; i++) {
			// Get the name
			var name = parameterNames[i];
			// Get the value
			var value = parameterValues[i];
			// Create the option
			var option = document.createElement("option");
			option.setAttribute("value", name + this._static._SEPARATOR + value);
			option.innerHTML = name + ": " + value;
			// Select the first option
			if (i == 0) {
				option.setAttribute("selected", "true");
			}
			// Add the option to the list box
			listBox.appendChild(option);
		}
		
		parent.appendChild(listBox);
		return listBox;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return Element
 	 */	
	_createUIOKButton: function(parent) {
		// OK button
		var okButton = document.createElement("button");
		okButton.innerHTML = this.messages.msgButtonOK;
		okButton.style.width = "55px";
		okButton.style.margin = "0px 3px 10px 128px";		
		this.connect(okButton, "onclick", this._handleEventOKButtonClicked);
		parent.appendChild(okButton);
		return okButton;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return Element
 	 */	
	_createUICancelButton: function(parent) {
		// Cancel button
		var cancelButton = document.createElement("button");
		cancelButton.style.width = "55px";
		cancelButton.style.margin = "0px 0px 10px 0px";	
		cancelButton.innerHTML = this.messages.msgButtonCancel;
		this.connect(cancelButton, "onclick", this._handleEventCancelButtonClicked);
		parent.appendChild(cancelButton);
		return cancelButton;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventOKButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Set positive flag
		this._doIt = true;
		// Get the selected option from the list box
		var selectedOption = new String(this._listBox.value);
		// Parse the name and value
		// Format <name><separator><value>
		var parsedData = selectedOption.split(this._static._SEPARATOR, 2);
		// Set name and value
		if (parsedData.length == 2) {
			this._selectedParameterName = parsedData[0];
			this._selectedParameterValue = parsedData[1];
		} else {
			alert(dojo.string.substitute(this.messages.msgInvalidOptionFormat, [selectedOption]));
		}
		
		// Hide the dialog
		this.hide();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Hide the dialog
		this.hide();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ParameterNameChooserDialog";
		
		this._static._SEPARATOR = "~";
		
		this._static._DEFAULT_LIST_SIZE = 5;
		this._static._MAX_LIST_SIZE = 15;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}		
	
});

})();
