/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative	= net.jazz.ajax.toolbox.lang.checkNonNegative;
var WorkItemUtil		= com.ibm.team.workitem.web.client.util;
var ImagePool			= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var MAX_SECTION_SIZE= 7;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView",
	dijit._Widget,
	{
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},
		postCreate: function() {
			dojo.subscribe(EventTopics.WORKITEM_EDITOR_CHANGED, this, this._checkWorkItemLinks);
			this.rows = [];
			this.childSubsections =[];
			this.filters = {};
			this.sectionId = 0;
			this.sectionCount = 0;
			this.sectionSize = MAX_SECTION_SIZE;
		},
		/* 
		 * A child subsection uses this objects data store 
		 */
		addChildSubsection: function(recentWorkItemView) {
			recentWorkItemView.setParentView(this);
			this.childSubsections.push(recentWorkItemView);
			recentWorkItemView.sectionId = this.childSubsections.length;
		},
		setParentView: function(parentRecentWorkItemView) {
			this.isChildView = true;
			this.parentView = parentRecentWorkItemView;
		},
		setFilter: function(filter)
		{
			dojo.mixin(this.filters, filter);
		},
		getRows: function() {
			if(this.isChildView) {
				return this.parentView.rows;
			} else {
				return this.rows;
			}
		},
		_checkWorkItemLinks: function(workItem) {
			if( !workItem || !this.parentController 
				|| !this.parentController._workItemPageView
				|| !this.parentController._workItemPageView._multipaneContentWidget			
				) return;

			var currentWidget = this.parentController._workItemPageView._multipaneContentWidget.getCachedWidget("__jazzWorkItemEditor", workItem.id);
			if(!currentWidget || !currentWidget.isWorkItemEditor || ! currentWidget._workItem) return;
			var rows = this.getRows();
			for(var i=0;i<rows.length;i++) {
				if (rows[i].id == (currentWidget._workItem.id)) {
					if (currentWidget._isDirty && rows[i].saveLink) {
						this._showSaveLink(rows[i].saveLink,true);
					}
					var url = currentWidget.getURL();
					if (rows[i].url != url && rows[i].node && rows[i].node.firstChild) {
						rows[i].node.firstChild.href = url;
					}
					break;
				}
			}
		},
		_isRowUnsaved: function(row) {
			if(row.isDirty) return true;
			if(row.id < 0) return true;
			return false;
		},		
		_showSaveLink: function(node, isShown) {
				var nodeArgs = {
					node: node,
					duration: isShown ? 800 : 10
				};
			
			if (dojo.isIE) {
				if (isShown) {
					node.style.display = "";
				}
				else {
					node.style.display = "none";
				}
			}
			else {
				if (isShown) {
					this._saveAnimation = dojo.fadeIn(nodeArgs);
					dojo.fx.combine([this._saveAnimation]).play();
				}
				else {
					if (this._saveAnimation) 
						this._saveAnimation.stop();
					node.style.opacity = "0";
				}
			}
				
		},
		/* 
		 * For any row that has a save parameter, update the timestamp
		 */
		updateSaveParameters: function() {
			var rows = this.getRows();
			for(var i=0;i<rows.length;i++ ) {
				if(rows[i].url && rows[i].saveLink) {
					var url = rows[i].url.replace(/&doSave=\([0-9]+\)/, "");
					rows[i].saveLink.href = url + "&doSave="+WorkItemUtil.getUniqueId();
				}
			}
		},
		_getRowNodeText: function(row) {
			var itemName = row.label;
			if(itemName.length > 500)
				itemName = itemName.substring(0,500) + "...";
			itemName = WorkItemUtil._breakupAllLongWords(itemName, 20);
			if(row.isDirty && row.id > 0 && itemName.indexOf("*") === -1) itemName = itemName + " *";
			return itemName;				
		},
		_showRow: function(index, isSelected) {
			var rows = this.getRows();
			var row = rows[index];
			if (!row.node || row.staleNode) {
				// compose name
				var itemName = this._getRowNodeText(row);
				
				// compose url
				var url = row.url.replace(/&doSave=\([0-9]+\)/, "");
				
				// determine where to insert the row
				for (var i = index + 1; i < rows.length; i++) {
					if (rows[i].node && rows[i].sectionId === this.sectionId) {
						var refNode = rows[i].node;
						break;
					}
				}
				var oldNode = row.node;
				
				row.node = this.navSection.addSectionRow(itemName, url, row.iconUrl, refNode);
				if(row.summaryLabel && row.node) {
					row.node.title = row.summaryLabel;
				} 
				row.saveLink = this.navSection.addArtifactCommand(row.node, this.messages.buttonSave, url + "&doSave="+WorkItemUtil.getUniqueId(), "save-workitem-command", true);
				
				if (oldNode && row.staleNode)
					this._disconnectNode(oldNode);
				row.staleNode = false;
			}
			// show save link if item is dirty
		   this._showSaveLink(row.saveLink,row.isDirty);
			
			if(isSelected)
				dojo.addClass(row.node,"selected");
			else
				dojo.removeClass(row.node,"selected");
			
		},
		_showQueryRow: function() {

			var info =  this.parentController.getLastQueryInfo();
			if(!info || !info.name || !info.isQuery) return;
			
			// compose name

			info.name =WorkItemUtil.getLeftNavString(info.name);
			var oldQueryRow = this.queryRow;
			var refNode = this.queryRow;
			var rows = this.getRows();
			for (var i = 0; !refNode && i < rows.length; i++) {
				if (rows[i].node && rows[i].sectionId === this.sectionId)
					refNode = rows[i].node;
			}
			
			this.queryRow = this.navSection.addSectionRow(info.name, info.url.replace(/&preserveFilter=true/, ""), ImagePool.WORKITEM_QUERY, refNode);
			this.queryRow.title = this.messages.titleShowMostRecentQueryResults;
			var artifactCommand = this.navSection.addArtifactCommand(this.queryRow, this.messages.titleRunMostRecentQueryResults , WorkItemUtil.reuniquifyURL(info.url, true), "refresh-query-command",/* hide until rollover */ true);
			if (this._queryRowEvent)
				this.disconnect(this._queryRowEvent);
			if (this._refreshCommandEvent)
				this.disconnect(this._refreshCommandEvent);
			if (this._staticQueryEvent)
				this.disconnect(this._staticQueryEvent);
			var listener = {
				self: this,
				setExplicit: function(/* boolean */ highlight) {
					this.self.isExplicit = info.itemId;
					if (highlight)
						this.self._doHighlighting();
				},
				clearExplicit: function() {
					this.self.isExplicit = false;
				}
			};
			this._queryRowEvent = this.connect(this.queryRow,"onclick",dojo.hitch(listener,listener.setExplicit, true));
			this._refreshCommandEvent = this.connect(artifactCommand,"onclick",dojo.hitch(listener,listener.setExplicit, false));
			var node = document.getElementById(info.itemId + "-QUERY");
			if (node)
				this._staticQueryEvent = this.connect(node,"onclick",dojo.hitch(listener, listener.clearExplicit));
			
			this._disconnectNode(oldQueryRow);
			return;
		},
		_showRows: function(showChildren) {
			showChildren = showChildren != false;
			
			// TODO: Don't fail if more than one section shows queries
			if(this.filters.showQuery) {
				this._showQueryRow();
			}
			var sectionCount = this.queryRow ? 1 : 0;
			
			var querySelected = this._doHighlighting();

			var rows=this.getRows();
			if(rows.length > 0) {
				var info =  this.parentController.getLastQueryInfo();
				var showRow;
				for(var i =0; i< rows.length; i++ ) {
//					if(rows[i].node) continue; // row already shown
					showRow = false;
					var isDirty = this._isRowUnsaved(rows[i]);
					if (((this.filters.showUnsaved &&  isDirty) ||
						 (this.filters.showSaved   && !isDirty))) {
						 	showRow = true;
						 }
					if( showRow && ((!this.filters.showUnsaved &&  isDirty) ||
						(!this.filters.showSaved   && !isDirty)))  {
						 	showRow = false;
						}
											
					if(showRow) {
						sectionCount++;
						var isSelected = false;
						// if the first on the list and showing the work item
						if (!querySelected && info.currentWorkItem && info.currentWorkItem.id == rows[i].id) {
							isSelected = true;
						}
						if(rows[i].node && rows[i].sectionId !== this.sectionId) {
							this._disconnectNode(rows[i].node);
							rows[i].node = null;
							rows[i].sectionId = -1;
						}

						var editor, wiType, wiIconURL, isTypeChanged;
						try {
							wiType = this.parentController.pageController.getCachedWorkItemEditor(rows[i].id).getShortLabel();
							wiIconURL = this.parentController.pageController.getCachedWorkItemEditor(rows[i].id).getIconURL();

						} catch(e) {}
						if(wiType){
							if(rows[i].isDirty && rows[i].id > 0 && wiType.indexOf("*") == -1) wiType = wiType + " *";
							if(rows[i].label != wiType) {
								isTypeChanged = true;
								rows[i].label = WorkItemUtil.plainToHTML(wiType);
							}
						}
						
						if (!rows[i].node) {
							if(isSelected || sectionCount < this.sectionSize) {
								this._showRow(i, isSelected);
							}
						} else {
							var node = rows[i].node;
							try {
							if(isTypeChanged) {
								for(var j=node.firstChild.childNodes.length-1; j>=0; j--) {
									if(dojo.hasClass(node.firstChild.childNodes[j],"link-text")) {
										dojox.data.dom.replaceChildren(node.firstChild.childNodes[j], document.createTextNode(wiType));
									} else {
										if(dojo.hasClass(node.firstChild.childNodes[j],"icon")) {
											node.firstChild.childNodes[j].firstChild.src = wiIconURL;
									}
									}
								}
							}
							} catch(e) {}; 
							if(isSelected) {
								dojo.addClass(rows[i].node,"selected");
								rows[i].node.style.display = "";								
							} else {
								dojo.removeClass(rows[i].node,"selected");
								if (sectionCount < this.sectionSize) {
									rows[i].node.style.display = "";
								} else {
									rows[i].node.style.display = "none";
								}
							}
						}
					
						rows[i].sectionId = this.sectionId;
					} else {
						if(rows[i].sectionId === this.sectionId) {
							this._disconnectNode(rows[i].node);
							rows[i].node = null;
							rows[i].sectionId = -1;
						}
					}
				}
			}
			
			if(sectionCount === 0 ) {
				dojo.addClass(this.navSection.domNode,"net-jazz-ajax-hidden");
			} else {
				dojo.removeClass(this.navSection.domNode,"net-jazz-ajax-hidden");
				var node = this.navSection._togglerLinkText || this.navSection._titleText; 
				if(node) {
					var sc = this.hasSectionCount ? " (" + sectionCount + ")" : "";
					dojox.data.dom.replaceChildren(node,document.createTextNode(this.heading + sc));
				}				
			}
			this.sectionCount = sectionCount;
			
			if (showChildren) {
				this.navSection.update(); // showChildren is overloaded
				if (!this.isChildView) {
					for (var i = 0; i < this.childSubsections.length; i++) {
						this.childSubsections[i]._showRows();
					}
				}
			}
		},
		_doHighlighting: function() {
			var info = this.parentController.getLastQueryInfo();
			if (this._selectedRow) {
				dojo.removeClass(this._selectedRow, "selected");
				this._selectedRow = null;
			} else {
				if(!info.isQuery && this.queryRow) {
					dojo.removeClass(this.queryRow, "selected");
				}
			}
			
			if (info.isQuery) {
				if (this.isExplicit != info.itemId)
					this.isExplicit = false;
				if (!this.isExplicit)
					var node = document.getElementById(info.itemId + "-QUERY");
				if (node) {
					this._selectedRow = node;
					dojo.addClass(this._selectedRow, "selected")
				} else if (this.queryRow)
					dojo.addClass(this.queryRow, "selected");
				return true;
			}
			return false;
		},
		addItem: function(id, label, url, isDirty, summaryLabel, iconUrl) {
			if(!label) return; // not initialized yet
			if(!isDirty) isDirty = false; // not null
			label = WorkItemUtil.plainToHTML(label);

			var rows=this.getRows();
			var item = {id: id, label: label, url: url, isDirty: isDirty, summaryLabel: summaryLabel, iconUrl: iconUrl};
			var idString = id + "";
			for(var i =0; i< rows.length; i++ ) {
				if(rows[i].id === id || (rows[i].id + "") == idString ) {
					var exists = true;
					rows[i].lastVisit = new Date().getTime();
					rows[i].isDirty = isDirty;
					break;	
				}
			}
			if (!exists) {
				item.lastVisit = new Date().getTime();
				rows.splice(0, 0, item);
			}
			this._showRows();
		},
		removeItem: function(id){
			var rows=this.getRows();
			for(var i =0; i< rows.length; i++ ) {
				if(rows[i].id == id + "") {
					this._disconnectNode(rows[i].node);
					rows.splice(i,1);
				}
			}
			this._showRows();
		},
		touchItem: function(id, label, url, isDirty, summaryLabel) {
			if(id) {
				this.addItem(id, label, url, isDirty, summaryLabel);
			} else {
				this._showRows();
			}
		},
		getOldestNonDirtyId: function(notThisId, cacheSize) {
			var oldestId = null;
			var oldestTime = new Date().getTime();
			var rows=this.getRows();
			var nonDirtyCount = 0;
			for(var i = 0; i < rows.length; i++) {
				if(rows[i].id > 0 && !rows[i].isDirty) {
					nonDirtyCount ++
					if(rows[i].id != notThisId && oldestTime > rows[i].lastVisit) {
						oldestId = rows[i].id;
						oldestTime = rows[i].lastVisit;
					}
				}
			}
			if(nonDirtyCount < cacheSize) return null;
			return oldestId;
		},
		_disconnectNode: function(/* domNode */ node) {
			if (node && node.parentNode) {
				node.parentNode.removeChild(node);
			}
		},
		showNextView: function() {
			var rows=this.getRows();
			if(rows && rows.length > 0) {
				this.selectedRow = rows[0];
				window.location.href = rows[0].url;
				return true; // unreachable;
			}

			if(this.queryRow && this.queryRow.childNodes[0]) {
				this.selectedRow = this.queryRow;
				window.location.href = this.queryRow.childNodes[0].href;
				return true; // unreachable
			}
			return false;
		},
		hasShowPartial: function() {
			return this.sectionCount > MAX_SECTION_SIZE;
		},
		
		onShowPartial: function() {
			this.sectionSize = MAX_SECTION_SIZE;
			this._showRows(/* showChildren */ false);
		},
		
		onMaximize: function() {
			this.sectionSize = 9999;
			this._showRows(/* showChildren */ false);
		}
	}
);

})();