/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that shows the Approvals Section
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<div class="ApprovalsPart SectionPart">' +
						'<span dojoAttachPoint="_sectionMenuSPAN"></span>' +
						'<div dojoAttachPoint="_approvalsDIV"></div>' +
						'</div>',

		postCreate: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");				
			this._approvalWidgets = [];
			this._approvalsDIV.innerHTML = this.messages.busyLoading; /* innerHTML OK */
			this._approvalsDIV.style.color = "#CCCCCC";
			this._isInitialized = false;

		},
		destroyWidget: function() {
			this.resetWidget();
			this.destroy();
		},
		resetWidget: function() {
			this.parentController.pageController.removeAllMouseListeners(this);
			for(var i = 0; i< this._approvalWidgets.length; i++) {
				this._approvalWidgets[i].destroyWidget();
			}
			this._approvalWidgets = [];

		},
		/*
		 * Lazy load the tab when it is shown
		 */
		showWidget: function() {
			if(this._isInitialized) return;
			this._isInitialized = true;
			setTimeout(dojo.hitch(this, this.refresh), 100);
		},
		
		/*
		 * Setup the page and read the states and types 
		 */
		refresh: function() {
			this.resetWidget();
			this._approvals = this.workItem.approvals;

			this._approvalsDIV.innerHTML = this.messages.busyLoading;
			this._approvalsDIV.style.color = "#CCCCCC";
			
			this._areTypesLoaded = false;
			this._areStatesLoaded = false;
			// Get the types from the server and update the child widget
			var fetchArgs = {
				serviceSelector: "getApprovalTypes",
				responseObject: this,
				responseSelector: "_typesLoaded",
				progressText: this.messages.busyLoadingApprovals
			};
			
			this.parentController.fetchSimpleInfo(fetchArgs);
			this._getApprovalStates();
			
		},
		_getApprovalStates: function(approvalTypes) {			
			// Get the state from the servers
			var fetchArgs = {
				serviceSelector: "getApprovalStates",
				responseObject: this,
				responseSelector: "_statesLoaded",
				progressText: this.messages.busyLoadingApprovals
			};
			
			this.parentController.fetchSimpleInfo(fetchArgs);
		},
		_statesLoaded: function(approvalStates) {
			this._approvalStates = approvalStates || [];
			this._areStatesLoaded = true;
			this._refreshApprovals();
		},
		_typesLoaded: function(approvalTypes) {
			this._approvalTypes = approvalTypes || [];
			this._areTypesLoaded = true;
			this._refreshApprovals();
		},
		_refreshApprovals: function() {
//			var start = new Date().getTime();
			if(!this._areTypesLoaded || !this._areStatesLoaded) return;

			this._approvalsDIV.style.color = "";

			// Can be called on initializtion, or on deleted approval
			var approvalStates = this._approvalStates || [];
			var approvalTypes = this._approvalTypes || [];
			this._approvals = this._approvals || [];
			this._approvalWidgets = [];
			
			this._createSectionMenu(approvalTypes);

			/*
			 * The states and types do not change, so 
			 * keep the actions as global.
			 */
			if (!this.stateActions || this.stateActions.length == 0) {
				this.stateActions = [];
				for (var i = 0; i < this._approvalStates.length; i++) {
					var state = approvalStates[i];
					var action = {
						isEnabled: true,
						receiverObject: this,
						receiverSelector: "changeApprovalState",
						label: state.label,
						iconSrc: state.iconUrl,
						value: state
					}
					this.stateActions.push(action);		
				}
			}

			if (!this.typeActions || this.typeActions.length == 0) {
				this.typeActions = [];
				for (var i = 0; i < this._approvalTypes.length; i++) {
					var type = this._approvalTypes[i];
					var action = {
						isEnabled: true,
						receiverObject: this,
						receiverSelector: "changeApprovalType",
						label: type.label,
						iconSrc: type.iconUrl,
						value: type
					}
					this.typeActions.push(action);
				}
			}
			
			var approvalsCount = 0;
			var deletedApprovalsCount = 0;
			var pendingApprovalsCount = 0;
			var approvedApprovalsCount = 0;
			var rejectedApprovalsCount = 0;
			if(this._approvals ) {
				for (var i = 0; i < this._approvals.length; i++) {
					this._approvals[i].localId = i;
					approvalsCount++;
					if(this._approvals[i]._isDeleted) {
						if(!this._approvals[i]._isNew) {
							deletedApprovalsCount++;
						}
					}
					if (this._approvals[i].cumulativeState == "com.ibm.team.workitem.approvalState.pending") {
							pendingApprovalsCount++;
					} else {
						if (this._approvals[i].cumulativeState == "com.ibm.team.workitem.approvalState.rejected") {
							rejectedApprovalsCount++;
						}
						else {
							if (this._approvals[i].cumulativeState == "com.ibm.team.workitem.approvalState.approved") {
								approvedApprovalsCount++;
							}
						}
					}
				}
			}
			
			var prefix = "";

			var pendingMsg = pendingApprovalsCount ? dojo.string.substitute(this.messages.msgApprovalPending, [pendingApprovalsCount]) : "";
			prefix = pendingApprovalsCount ? prefix = ", " : "";

			var approvedMsg = approvedApprovalsCount ? prefix + dojo.string.substitute(this.messages.msgApprovalApproved, [approvedApprovalsCount]) : "";
			prefix = approvedApprovalsCount || pendingApprovalsCount ? prefix = ", " : "";
			
			var rejectedMsg = rejectedApprovalsCount ? prefix + dojo.string.substitute(this.messages.msgApprovalRejected, [rejectedApprovalsCount]) : "";

			prefix = pendingApprovalsCount || approvedApprovalsCount || rejectedApprovalsCount ?  prefix = " - " : "";
			
			if ((approvalsCount) == 0) {
				this.sectionLayout.sectionTitle.style.fontWeight = "";
				this.sectionLayout.sectionTitle.innerHTML = this.messages.headingApprovals;
				this._approvalsDIV.innerHTML = "<br>&nbsp;&nbsp;&nbsp;&nbsp;" + this.messages.msgNoApprovals;
				return;
			}
			else {
				this.sectionLayout.sectionTitle.style.fontWeight = "normal";
				this.sectionLayout.sectionTitle.innerHTML = "<b>" + dojo.string.substitute(this.messages.msgApprovalCount, [approvalsCount]) + "</b>" + prefix + pendingMsg + approvedMsg + rejectedMsg;
			}
						
			dojox.data.dom.removeChildren(this._approvalsDIV);
			
			if (!this._heading) {
				// Create a heading to show column titles
				var creationArgs = {
					parentWidget: this,
					parentController: this.parentController,
					parentWidget: this,
					isHeading: true,
					isFirst: true,
					className: "SectionListTableHeaderRow"
				};
				var approvalWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget(creationArgs);	
			} else {
				approvalWidget = this._heading;
			}
			this._approvalWidgets.push(approvalWidget);
			this._approvalsDIV.appendChild(approvalWidget.domNode);

			var listDiv = document.createElement("DIV");
			for(var i=this._approvals.length-1;i>=0;i--) {
				var isDeleted = this._approvals[i]._isDeleted || false;
				if(this._approvals[i].widget && (isDeleted != this._approvals[i].widget._isDeleted)) {
					this._approvals[i].widget.destroy();
					this._approvals[i].widget = null;
				}
				if (!this._approvals[i].widget) {
					creationArgs = {
						isFirst: i == 0,
						parentWidget: this,
						parentController: this.parentController,
						approval: this._approvals[i],
						_isNew: this._approvals[i]._isNew || false,
						_isDeleted: isDeleted,
						approvalTypes: this._approvalTypes,
						approvalStates: this._approvalStates
					};
					
					approvalWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget(creationArgs);
					this._approvalWidgets.push(approvalWidget);
					this._approvals[i].widget = approvalWidget;
				} else {
					approvalWidget = this._approvals[i].widget;
				}
				listDiv.appendChild(approvalWidget.domNode);
			}
			// Create a blank footer
			if (!this._footer) {
				var creationArgs = {
					parentWidget: this,
					parentController: this.parentController,
					isHeading: true,
					isFooter: true
				};
				approvalWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget(creationArgs);
				this._approvalWidgets.push(approvalWidget);
			} else {
				this._approvalWidgets.push(this.footer);
				approvalWidget = this._footer
			}
			listDiv.appendChild(approvalWidget.domNode);

			this._approvalsDIV.appendChild(listDiv);

//			console.debug("4: " + (new Date().getTime() - start))

		},
		_getTargetLabel: function() {
			var targetLabel = null;
			try {
			  targetLabel = this.parentController._editorParts.overview.allWidgets.target.getSelectedLabel();
			} catch(e) {} // ignore


			if(!targetLabel && this.workItem && this.workItem.attributes.target && this.workItem.attributes.target.label) {
				targetLabel = this.workItem.attributes.target.label;
			}
			
			if(!targetLabel || targetLabel.length == 0 || targetLabel.toLowerCase() == "unassigned" || targetLabel.toLowerCase() == "unknown") {
					targetLabel = ""; // TODO: Return the date
			}
			targetLabel = targetLabel ? targetLabel.replace(/ /g,"_") : "";
			return targetLabel;
		},
		// The default approval name is:
		//	Target ApprovalType (n)
		_getDefaultName: function(approval) {
			
			var targetLabel = this._getTargetLabel();
			
			
			var typeLabel = this._getTypeLabel(approval);
			
			var prefix = targetLabel.length > 0 ? targetLabel + "_" + typeLabel : typeLabel;
			
			// Make sure there are no other approvals with this label
			var result = prefix;
			var suffixCount = 2
			while(this._hasApprovalWithName(result, approval)) {
				result = prefix + dojo.string.substitute(this.messages.msgParenCount, [ suffixCount++ ]);
			}
			return result;
		},
		removeApproval: function(approval) {
			for(var i=0;i<this._approvals.length; i++ ) {
				if(this._approvals[i].localId == approval.localId) {
					if(this._approvals[i].widget)
						this._approvals[i].widget.destroy();
					this._approvals.splice(i,1);
					break;
				}
			}
		},
		_getTypeLabel: function(approval){
			var typeId = approval.approvalType;
			var typeLabel = this.messages.msgApproval;
			for(var i=0;i<this._approvalTypes.length;i++ ) {
				if(typeId == this._approvalTypes[i].id)	{
					typeLabel = this._approvalTypes[i].label;
					break;
				}
			} 
			return typeLabel;
		},
		_hasApprovalWithName: function(name, approval) {
			if(!this._approvals || !approval) return false;
			for(var j=0;j<this._approvals.length; j++) {
				if(this._approvals[j].name == name && approval.localId != this._approvals[j].localId) return true;
			}
			return false;
		},
		_getUpdatedDefaultName: function(approval) {
			try{
			
			var targetLabel = this._getTargetLabel();	
				
				var prefix = this._getDefaultName(approval);
				if(!approval || !approval.name) return prefix;
				var spaceIndex = approval.name.indexOf(" ");
				var colonIndex = approval.name.indexOf(":");
				if(spaceIndex == -1 && colonIndex == -1) {
					
					if(!this._isDefaultName(targetLabel, approval.name)) return null;
					
					return prefix;
				}
				
				var searchString;
				if (colonIndex == -1) {
					searchString = " ";
				}
				else {
					if (spaceIndex == -1) {
						searchString = ":";
					}
					else {
						searchString = spaceIndex < colonIndex ? " " : ":";
					}
				}
				
				var firstWord = approval.name.split(searchString)[0];
				
				// The first word must contain a type and target or we leave it alone
		
				if(!this._isDefaultName(targetLabel, firstWord)) return null;
				
				
				return approval.name.replace(firstWord,prefix);	
			} catch(e) {
				// ignore
			}
			return null;
		},
		_isDefaultName: function(targetLabel, name) {
				if(name.indexOf(targetLabel)== -1) return false;
				
				var found = false;
				for(var i=0;i<this._approvalTypes.length;i++ ) {
					var typeLabel = this._approvalTypes[i].label;
					if(name.indexOf(typeLabel) != -1) {
						found = true;
						break;	
					}
				}
				return found
		},
		/*
		 * Build the add approval menu  
		 */
		_createSectionMenu: function(approvalTypes) {
			if(this.parentController.pageController.isGuest()) return;
			if(this.sectionMenuWidget) {
				dojox.data.dom.removeChildren(this._sectionMenuSPAN);
				this.sectionMenuWidget.destroyWidget();
			}
			
			var addActions = [];
			for(var i=0;i<approvalTypes.length;i++){
				var type = approvalTypes[i];
				var action = {
					isEnabled: true,
					receiverObject: this,
					receiverSelector: "addNewApproval",
					label: type.label,
					iconSrc: type.iconUrl,
					value: type
				}
				addActions.push(action);
			}
			var actionNode = document.createElement("SPAN");
			var creationArgs = {
						parentWidget: this.parentWidget,
						actions: addActions,
						label: this.messages.buttonNew
					};
			var addActionWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget(creationArgs, actionNode);
			
			creationArgs = {
						parentWidget: this.parentWidget,
						actionWidgets: [ addActionWidget ]
					};
			
			this.sectionMenuWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget(creationArgs, this._sectionMenuSPAN);

		},
		/*
		 * Add a new approval when the action is clicked from the section menu
		 */
		addNewApproval: function(action) {
			this.parentWidget.parentController.markDirty();
			if(!this._approvals) {
				this._approvals = [];
			}
			
			var approval = {
				approvalType: action.value.id,
				cumulativeState: "com.ibm.team.workitem.approvalState.pending",
//				dueDate: null,
//				id: null,
				_isNew: true
			}
			approval.name = this._getDefaultName(approval);
			this._approvals.push(approval);
			
			this._refreshApprovals();
		},
		

		//
		// Part API Metnods
		//
		

		setReadOnly: function(isReadOnly) {
//			this.inputWidget.setReadOnly(isReadOnly);
		},
		getFormElementValues: function(formElements) {
//			if(this.inputWidget.isChanged) {
//				var value = this.inputWidget.getValue();
//				if(value == "") value = -1;
//				var valueFloat;
//				valueFloat = parseFloat(value);
//				if(valueFloat.toString() == "NaN" || typeof valueFloat != "number" || valueFloat < 0) {
//					value = -1;
//				} else {
//	
//				}
//				formElements[this.attributeId] = value;
//			}
		},
		setFormElementValues: function(retainedState){
//			var value = retainedState[this.attributeId];
//			retainedState[this.attributeId] = value;
//			this.inputWidget.setFormElementValues(retainedState);
			},
		/*
		 * Return an array  of commands. See work item 40464.
		 * 
		 * Command Format is:
		 * 
		 *      Create a new approval request for a specific user
		 *		var approvalType= ... // e.g. getApprovalTypes()[0].id
		 *		var contributorItemId= ...
		 *		var command= { cmd: "createApproval", param: { type: approvalType, name: "M5 Approval", approvers: [ { user: contributorItemId } ] }};
		 *		
		 *		Update an approval descriptor with another name and dueDate
		 *		var approvalId= ...// The id of an approval descriptor, e.g. workItem.approvals[0].id
		 *		var date= ...
		 *		var command= { id: approvalId, cmd: "updateApproval", param: { name: "New name", dueDate: date.getTime() } };
		 *		
		 *		Add a user to an existing approval request
		 *		var contributorItemId= ...
		 *		var approvalId= ...// The id of an approval descriptor, e.g. workItem.approvals[0].id
		 *		var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId } };
		 *		
		 *		Change an approval state
		 *		var contributorItemId= ...
		 *		var state= ... // e.g. getApprovalStates()[0].id
		 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
		 *		var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId, state: state } };
		 *		
		 *		Remove an approver
		 *		var contributorItemId= ...
		 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
		 *		var command= { id: approvalId, cmd: "removeApprover", param: { user: contributorItemId } };
		 *		
		 *		Remove an approval descriptor
		 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
		 *		var command= { id: approvalId, cmd: "removeApproval" };
		 *
		 *    A removed approval stays on the approvals list but is not shown, it returns its own
		 *    removal command
		 *
		 */ 
		 getSaveCommands: function (allCommands) {
		 	if(!this._approvals) return allCommands;
			
		 	var commands = [];
			for(var i=0; i<this._approvals.length; i++) {
				var approval = this._approvals[i];
				if(!approval) continue;
				if(approval._isNew) {
					if (!approval._isDeleted) {
						// For a new approval, consolidate all commands into one command
						var command = {
							cmd: "createApproval",
							param: {
								type: approval.approvalType,
								name: approval.name
							}
						}
						if (approval.dueDate) {
							command.param.dueDate = approval.dueDate;
						}
						if (approval.approvals) {
							command.param.approvers = [];
							for (var j = 0; j < approval.approvals.length; j++) {
								subapproval = approval.approvals[j];
								if (subapproval && subapproval.approver) {
									var approverCommand = {
										user: subapproval.approver.itemId,
										state: subapproval.state
									}
									command.param.approvers.push(approverCommand)
								}
							}
						}
						commands.push(command);
					}
				} else {
					// For approval updates, send all the commands normally
					if (approval._commands) {
						for (var j = 0; j < approval._commands.length; j++) {
							if (approval._commands[j]) {
								commands.push(approval._commands[j]);
							}
						}
					}
					if (approval.approvals) {
						for (var j = 0; j < approval.approvals.length; j++) {
							var subapproval = approval.approvals[j];
							if (subapproval && subapproval.approver && subapproval.approver._command) {
								commands.push(subapproval.approver._command);
							}
						}
					}
					
				}
			}
			if(commands.length > 0 ) {
				allCommands.updateApprovals = commands;
			} 
			return allCommands;
		 },
	     getValue: function() {
//			return this.inputWidget.getValue();
		 }
	}
);

})();
