/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioGroupPart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var ImagePool 	= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

/* Define RadioButton Widget that supports icon and label */ 
dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioButton",
	dijit.form.RadioButton,
	{
		templateString: "<div><fieldset dojoAttachPoint='stateNode' class=\"dijitReset dijitInline radioalign\" waiRole=\"presentation\"><input dojoType=\"dijit.form.RadioButton\" type=\"radio\" name=\"${name}\" value=\"${value}\" class=\"dijitReset dijitCheckBoxInput\" dojoAttachPoint=\"inputNode,focusNode\" dojoAttachEvent=\"onchange:_onChange, onclick:_onClick\"></Input></fieldset><label dojoAttachPoint=\"_label\"><img class=\"RadioButtonImage\" dojoAttachPoint=\"_image\" src=\"${icon}\"/><span class=\"radioalign\">${label}</span></label></div>",
		icon: "",
		_onChange: function(){
		}
	}
);

/* Define RadioGroup Part*/
dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioGroupPart",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="EnumerationWidget NarrowWidget" dojoAttachPoint="_radiogroupwidget">' +
						'<div dojoAttachPoint="_radiogroup">' +
						'</div>' +
						'<span  style="display: none"  dojoAttachPoint="_readOnlyRadioGroupValue" class="GreyLabel"></span>'+
						'</div>',						

		_radiobuttons: new Array(),
		_select: null,
		_id: null,
  		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function(){
			this._childWidgets = [];
			this._modifiers = this._modifiers ? this._modifiers : {};
			var value = this._bindValuesToRadioGroup(this._possibleValues, this._radiogroup);
			_id = value;
			this._setReadOnlyValue(value==null?null:value.label);
			
			if(this._modifiers && this._modifiers.onCreate && this.parentWidget[this._modifiers.onCreate]) {
					this.parentWidget[this._modifiers.onCreate](this, value);
			}
			if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget[this._modifiers.onValueChanged]) {
					this.parentWidget[this._modifiers.onValueChanged](this , value, /* isInitialization */ true);
				}
			if (this.parentController.pageController.isGuest()) {
				dojo.addClass(this._radiogroup, "GuestField");
			}
			this._isChanged = false;			
		},
		destroyWidget: function() {
			this.resetWidget();
			this.destroy();
		},
		resetWidget: function() {
			
			for(var i = 0; i< this._childWidgets.length; i++) {
				if( this._childWidgets[i].destroyWidget) {
					this._childWidgets[i].destroyWidget();	
				} else {
					if(this._childWidgets[i].destroy) {
						this._childWidgets[i].destroy();
					}
				}
			}
			this._childWidgets = [];
		},
		getValue: function() {
			var value = this._getValue();
			return value == null?null:value.id;
		},
		
		getSelectedLabel: function(){
			if(!this._radiogroup) return "";
			var value = this._getValue();
			return value.label;			
		},
		
		setReadOnly: function(isReadOnly) {
			if (isReadOnly) { // was mouse out
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				this._radiogroup.style.display = "none";
				this._readOnlyRadioGroupValue.style.display = "inline";
			}
			else {
				this._radiogroup.style.display = "inline";
				this._readOnlyRadioGroupValue.style.display = "none";
			}
		},		

		getFormElementValues: function(formElements) {
			var value = this._getValue();
			if (value != null && value.id != EditorUtil.UNCONFIGURED) {
				formElements[this.attributeId] = value.id;
			}			
		},

		setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
		 		var currentValue = this.getValue();
		 		if (currentValue != retainedState[this.attributeId]) {
					for(var i= 0; i<this._radiobuttons.length;i++){
						if(this._radiobuttons[i] && this._radiobuttons[i].value.id == retainedState[this.attributeId]){
							this._radiobuttons[i].setChecked(true);
							this._isChanged = true;
							break; 
						} 
					}
					var value = this._getValue();
					this._setReadOnlyValue(value==null?null:value.label);
				}
			}			
		},
		
		_clicked: function(e){
			var target = EditorUtil._getOriginalTarget(e);
			if(target.id != this._id)
				this._onChange(e);
		},
		
		_onChange: function(e) {
			this._isChanged=true;
			
			this._select = EditorUtil._getOriginalTarget(e);
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent(e);
			}

			var value = this._getValue();
			this._setReadOnlyValue(value.label);
			if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
				this.parentWidget[this._modifiers.onValueChanged](this , value, /* isInitialization */ false);
			}
			this._id = EditorUtil._getOriginalTarget(e).id;			
		},
		
		_enumHasValue: function(value){
			var found = false;
			if(value && value.id!=null) {
				for(var i=0;i<this._possibleValues.length;i++) {
					if(this._possibleValues[i].id == value.id) {
						found = true;
						break;
					}
				}
			}
			return found;		
		},
		
		_setReadOnlyValue: function(value){
			if(value == null) {
				if(this._modifiers.noValueLabel != null) {
					value = this._modifiers.noValueLabel;
				} else {
					value = this.messages.labelNotAvailable;
				}
			}
			dojox.data.dom.removeChildren(this._readOnlyRadioGroupValue)
			this._readOnlyRadioGroupValue.appendChild(document.createTextNode(value));
		},		

		_getValue: function(){
			var value = null;
			for(var i= 0; i<this._radiobuttons.length;i++){
				if(this._radiobuttons[i] && this._radiobuttons[i].checked){
					value = this._radiobuttons[i].value;
					break; 
				} 
			}
			return value == null?null:value;		
		},
		
		_wireRadioButton: function(button){
			if(button){
				button._label.htmlFor = button.inputNode.id;
				this._radiogroup.appendChild(button.domNode);
				if(dojo.isIE)
					this.connect(button.domNode, 'onclick', this._clicked);
				else
					this.connect(button.domNode, 'onchange', this._onChange);
				this._radiobuttons.push(button);
			}		
		},

		_bindValuesToRadioGroup: function(possibleValues, radioGroup) {
			this.resetWidget();
			var resultValue = this.attributeValue;
			this._id = null;
			this._radiobuttons = new Array();
			if (!possibleValues || possibleValues == null || possibleValues.length === 0) {			//Enum is empty
				if(!this.attributeValue || this.attributeValue == null || this.attributeValue.id==null || this.attributeValue.id=="unassigned") {
					this._setReadOnlyValue(null);
					this.setReadOnly(true);
				}else if(this.attributeValue.label!=null && this.attributeValue.id!=null){	//Value is present.
					dojox.data.dom.removeChildren(this._radiogroup)
					var button = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioButton({name: this.id, label: this.attributeValue.label, icon: this.attributeValue.iconUrl, value: this.attributeValue});
					this._childWidgets.push(button);
					button.setChecked(true);
					this._id = button.inputNode.id; 
					button.disabled = true;
					this._wireRadioButton(button);
				}
			}else{
				var hasAtleastOneImage = false;
				var emptyImage = WidgetUtil.getUnknownIconUrl();
				var orphanButton = null;
				var buttonsWithNoIcon = new Array();
				
				//Orphan Value
				if(this.attributeValue &&  !this._enumHasValue(this.attributeValue)){
					if(this.attributeValue.id != null && this.attributeValue.id !=""){
						orphanButton = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioButton({name: this.id, label: this.attributeValue.id, value: this.attributeValue});
						this._childWidgets.push(orphanButton);
						orphanButton.setChecked(true);
						this._id = orphanButton.inputNode.id;
						buttonsWithNoIcon.push(orphanButton);
						this._wireRadioButton(orphanButton);
					}
				}

				for (var i=0; i<possibleValues.length; i++) {
					var button = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioButton({name: this.id, label: possibleValues[i].label, value: possibleValues[i]});
					this._childWidgets.push(button);
					var image  = possibleValues[i].iconUrl;
					if(!hasAtleastOneImage){
						if(image != null && image!="")
							hasAtleastOneImage = true;
						else
							buttonsWithNoIcon.push(button);
					}
					if(hasAtleastOneImage){
						if(image == null || image =="")
							image = emptyImage;
						button._image.src = image;
					} else {
						button._image.src = ImagePool.PIXEL.uri;
					}
					
					//Select Value
					var selectedValueId=this.attributeValue ? this.attributeValue.id : "";
					if(selectedValueId && selectedValueId === possibleValues[i].id) {
						button.setChecked(true);
						resultValue = possibleValues[i];
						this._id = button.inputNode.id;
					}
					this._wireRadioButton(button);
				}
				if(hasAtleastOneImage){
					for(var i=0;i<buttonsWithNoIcon.length;i++){
						buttonsWithNoIcon[i]._image.src = emptyImage;
					}
				}
			}
			return resultValue;
		}
	}
);
})();
