/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="ContributorWidget NarrowWidget">' +
						'<img dojoAttachPoint="_warning" style="visibility:hidden" class="PreFieldImage" />' +
						'<span  style="display: none" dojoAttachPoint="_readOnlySelectValue" class="ReadOnlySelectValue"></span>' +						
						'<div class="fieldWrapper"><select style="display: none" dojoAttachPoint="_select" dojoAttachEvent="onchange: _handleSelectChange" class="Select"></select></div>' +
						'<span style="display: none" dojoAttachPoint="_anchorSPAN"></span>' +
						'<span style="display:none" dojoAttachPoint="_noValueLabelSPAN" class="LayoutValue GreyLabel"></span>' +
						'</div>',
     constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			this._warning.src = ImagePool.SMALLWARNING;
			this._warning.title = this.messages.titleOwnerDoesNotBelongToTeamArea;
			if(this._modifiers.readonly == "true") this._modifiers.readonly = true;
			
			if(this.creationSpec && this.creationSpec.isDescription) {
					dojo.addClass(this.domNode,"MarginLeft10");
				}
			
			if(this._modifiers.readonly) {
				this.createLabel();
			} else {
				this.createSelect();
			}
			this._isChanged = false;
			if(this.parentController.pageController.isGuest()) {
				dojo.addClass(this._select, "GuestField");
			}
		},
		setReadOnly: function(isReadOnly) {
			if(isReadOnly){ // was mouse out
				if(this._modifiers && this._modifiers.readonly) return;
				// Always show it if is changed
				if(this._isChanged) {
					this.setReadOnly(false);
					return;	
				}
				this._select.style.display = "none";
				this._readOnlySelectValue.style.display = "inline";
			}
			else {
				if(this._modifiers && this._modifiers.readonly) return;
				this._select.style.display = "inline";
				this._readOnlySelectValue.style.display = "none";
			}
		},
		createLabel: function() {
			var value;
			if(this.values && this.values.length > 0) {
				value = this.values[0];
			}  
			if(value.name == "Unassigned" && value.userId == "unassigned") {
				value.isNullValue = true;
			}
			
			if(!value || value.length == 0 || value.isNullValue) {
				value = this.messages.labelDefaultNullContributor;
				if(this._modifiers.noValueLabel) {
					value = this._modifiers.noValueLabel;
				}
				if(this._modifiers.noValueTitle) {
					this._noValueLabelSPAN.title = this._modifiers.noValueTitle;
				} 
				this._noValueLabelSPAN.style.display="inline";
				dojox.data.dom.replaceChildren(this._noValueLabelSPAN, document.createTextNode(value));
				return;
			} 
			var href = this._modifiers["href"];
			if(!href && value.emailAddress) {
					var emailSubject =  this._modifiers.emailSubject;
					var href = WidgetUtil._getMailto(value.emailAddress, emailSubject);
				}
				
			var name = value.name ? value.name : value.label;
			name = name ? name : labelDefaultNullContributor;
				
// If there is an href, add the anchor, otherwise just add a label
// <a dojoAttachPoint="_anchor" class="LayoutValue Anchor"><span dojoAttachPoint="_labelSPAN" class=""></span></a>
			var labelTarget = this._anchorSPAN;
			var labelClass = "LayoutContributorLabel";		
			if(href) {
				var anchor = document.createElement("A");
				anchor.href = href;
				dojo.addClass(anchor, "RelativeValue");
				dojo.addClass(anchor, "Anchor");
				this._anchorSPAN.appendChild(anchor);
				labelTarget = anchor;
				labelClass = "RelativeValue";
			}
			
			var labelSPAN = document.createElement("SPAN");
			labelSPAN.appendChild(document.createTextNode(name));
			if(labelClass) dojo.addClass(labelSPAN, labelClass);
			
			labelTarget.appendChild(labelSPAN);
			
			this._anchorSPAN.style.display="inline";
		},
		createSelect: function() {	
			var value = WidgetUtil._bindValuesToSelect(this, this._possibleValues, this._select, null,this._modifiers.moreLabel,this._modifiers.moreTitle);
			var props = WidgetUtil._getCurrentSelectProps(this._select);
			if(!value || !props) return;
			this._priorSelectedValueId = props.id;
			this._setReadOnlySelectValue(props.label);
//			this._readOnlySelectValue.style.display="inline";
			this._setOwnerWarningIconVisibility(value.id);
			if(this._modifiers && this._modifiers.onCreate && this.parentWidget && this.parentWidget[this._modifiers.onCreate]) {
					this.parentWidget[this._modifiers.onCreate](this , value);
				}
			this.setReadOnly(false);
		},
		_setReadOnlySelectValue: function(value){
			dojox.data.dom.removeChildren(this._readOnlySelectValue)
			this._readOnlySelectValue.appendChild(document.createTextNode(value));
		},
		updateOptions: function(newPossibleValues, newValueId) {
			this._possibleValues = newPossibleValues;
			var value = WidgetUtil._bindValuesToSelect(this, this._possibleValues, this._select, null,this._modifiers.moreLabel,this._modifiers.moreTitle);
			this._select.value = newValueId;
			if(newValueId) {
				this._priorSelectedValueId = newValueId;				
			}
			this._setOwnerWarningIconVisibility(newValueId);
			
			var label = "";
			try {
				label = this._getCurrentSelectProps(this._select).label;
			} catch(e) {}; // ignore
			this._setReadOnlySelectValue(label);

		},
		_setOwnerWarningIconVisibility: function(selectedValueId) {
			if(selectedValueId) {
				var needsWarning = false;
				for(var i=0;i<this._possibleValues.length;i++) { 
					if(this._possibleValues[i].id == selectedValueId) {
					    if(this._possibleValues[i].isOutOfProjectArea) {	needsWarning = i>0; };// Skip the "Unassigned" value
						break;		
					}
				}
				if(needsWarning){ 
					this._warning.style.visibility = "visible"; 
				} else {
					this._warning.style.visibility = "hidden";
				}
			}

		},
		getFormElementValues: function(formElements) {
//			if(!this._isChanged) return;
			/* if(this._select.value != "") */ formElements[this.attributeId] = this._select.value;
		}, 
		setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
				if(this._select.value != retainedState[this.attributeId]) {
		 			this._select.value = retainedState[this.attributeId];
					this._isChanged = true;
					var selectedValue = WidgetUtil._getCurrentSelectProps(this._select);
					if(selectedValue && selectedValue.label) {
						this._setReadOnlySelectValue(selectedValue.label);
					}
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , selectedValue);
					}
					this._setOwnerWarningIconVisibility(this._select.value);
				}
			}
		 },
		_handleSelectChange: function(e) {	
			this._isChanged=true;
			var select= EditorUtil._getOriginalTarget(e);
			var selectedValue = WidgetUtil._getCurrentSelectProps(select);
			this._setReadOnlySelectValue(selectedValue.label);
//			if(this._priorSelectedValueId) {
//				select.value = this._priorSelectedValueId;				
//			}
			if(selectedValue.id == -1) {// More...
				var functionName =this._modifiers.moreFunctionName; 
				this.parentWidget[functionName](this);
			}
			this.parentWidget._handleDirtyingEvent(e);
			this._setOwnerWarningIconVisibility(selectedValue.id);
			if (select === this._select) {
				if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
					this.parentWidget[this._modifiers.onValueChanged](this , selectedValue);
				}
			}
		}
	}
);

})();
