/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="EnumerationWidget NarrowWidget">' +
						'<img dojoAttachPoint="_image" class="PreFieldImage" />' +
						'<div class="fieldWrapper"><select dojoAttachPoint="_select" dojoAttachEvent="onchange: _handleSelectChange, onclick:_handleSelectClick, onfocus:_handleSelectFocus, onblur: _handleSelectBlur" class="Select"></select></div>' +
						'<span style="display: none" dojoAttachPoint="_readOnlySelectValue" class="ReadOnlySelectValue"></span>' +
						'</div>',
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			WidgetUtil.setSpacerImage(this._image);

			if(this.creationSpec && this.creationSpec.isDescription) {
					dojo.addClass(this.domNode,"MarginLeft10");
				}

			this._modifiers = this._modifiers ? this._modifiers : {};
			// If there are no possible values, but there is a value,
			// Add the value as a possible value, but always be disabled
			this._isAlwaysDisabled = false;
			if(this.creationSpec && this.creationSpec.isBlockedAttributePresentation) {
				this._isAlwaysDisabled = true;
			}
			if(this._possibleValues ==null || this._possibleValues.length == 0) {
				if(this.values && this.values.length > 0 ) {
					this._isAlwaysDisabled=true;
					this._possibleValues = [
						this.values[0] == "" ? "&nbsp;" : this.values[0]
					];
				}
			}
			
			if(this.creationSpec.isNotAvailable) {
				this._bindValuesToSelect([{label: this.messages.labelDuplicateAttributeNotAvailable , id: -2}], this._select, this._image);
				this._select.disabled = true;
				return;
			} 
				
			this._possibleValues = this._possibleValues || [];
			var value;
			
			// Set all versions of a null id to an empty string
			for (var i = 0; i < this._possibleValues.length; i++) {
			   	   if(this._possibleValues[i] && 
				   	 (this._possibleValues[i].id == null || 
					  this._possibleValues[i].id == "" || 
					  this._possibleValues[i].isNull  || 
					  this._possibleValues[i].id=="unassigned")) {
  					 	 this._possibleValues[i].id = "";
  					 	 this._possibleValues[i].isNull = true;
					  }
			}

			// if the attribute value is null, disable and return
			if(this.attributeValue == null) {
			  this._select.disabled = true;
			  return;
			};
			// Set all versions of a null id to null
			if(this.attributeValue.id == null || 
			   this.attributeValue.id == "" || 
			   this.attributeValue.isNull  || 
			   this.attributeValue.id=="unassigned"){
			   	 	 this.attributeValue.id = "";
  					 this.attributeValue.isNull = true;
			  }
			
			try {
				// if the value is not in possible values
				// then add the value as the first possible value
				if(this.attributeValue) {
					value = this.attributeValue;
					if(value) {
						var found = false;
						for(var i=0;i<this._possibleValues.length;i++) {
							if(this._possibleValues[i].id == value.id ) {
								found = true;
								break;
							}
						}
						if (!found) {
							this._possibleValues.unshift(value);
						}
					}
				}
			} catch(e) { // ignore for now 
			}

			// bindValuesToSelect will select this.attributeValue			
			value = this._bindValuesToSelect(this._possibleValues, this._select, this._image);
			
			var label = "";
			try{
				label = this._getCurrentSelectProps(this._select).label
			} catch(e) {}; // ignore
			this._setReadOnlySelectValue(label);
			if(this._modifiers && this._modifiers.onCreate) {
					this.parentWidget[this._modifiers.onCreate](this, value);
			}
			if(this._modifiers && this._modifiers.onValueChanged) {
					this.parentWidget[this._modifiers.onValueChanged](this , value, /* isInitialization */ true);
				}
			if (this.parentController.pageController.isGuest()) {
				dojo.addClass(this._select, "GuestField");
			}
			this._isChanged = false;
		},
		setReadOnly: function(isReadOnly) {
			if (isReadOnly) { // was mouse out
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				
				this._select.style.display = "none";
				this._readOnlySelectValue.style.display = "inline";
			}
			else {
				this._select.style.display = "inline";
				this._readOnlySelectValue.style.display = "none";
			}
		},
		_setReadOnlySelectValue: function(value){
			
//			value = value == null ? "Not Available" : value;
			if(value == null) {
				if(this._modifiers.noValueLabel != null) {
					value = this._modifiers.noValueLabel;
				} else {
					value = this.messages.labelDefaultNullEnumerationValue;
				}
			}
			 
			dojox.data.dom.removeChildren(this._readOnlySelectValue)
			this._readOnlySelectValue.appendChild(document.createTextNode(value));
		},
		_handleSelectChange: function(e) {
			this._isChanged=true;
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent(e);
			}
			var select= EditorUtil._getOriginalTarget(e);
			var selectValue = this._getCurrentSelectProps(select);
			this._setReadOnlySelectValue(selectValue.label);
			if(selectValue.iconUrl) {
				this._image.src = selectValue.iconUrl;
				this._image.style.visibility="visible";
			} else {
				WidgetUtil.setSpacerImage(this._image);
			}
			
			if (select === this._select) {
				if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
					for(var i = 0;i<this._possibleValues.length;i++) {
						if(this._possibleValues[i].id == selectValue.id) {
							selectValue.value =this._possibleValues[i];
							break; 
						}
					}					
					this.parentWidget[this._modifiers.onValueChanged](this , selectValue, /* isInitialization */ false);
				}
			}
			this._handleSelectFocus(e);// Use the short form of labels.
		},
		_handleSelectClick: function(e) {
		}, 
		_handleSelectFocus: function(e) {
			/* Special case for categories */
			if (dojo.isIE) return;
			try {
			if(!this._modifiers || !this._modifiers.allCategories) return;
			var select= EditorUtil._getOriginalTarget(e);
			var index  = select.selectedIndex;
			if(!index)return;
			var valueObject = this._possibleValues[index];
			if(!valueObject) return;
			if(select.options[index].__jazzWasClicked) return;
			select.options[index].__jazzWasClicked = true;
			var newLabel = valueObject.shortLabel;
			
			if(select.options[select.selectedIndex].childNodes[0] && 
			   select.options[select.selectedIndex].childNodes[0].nodeValue == newLabel) return;
			
			dojox.data.dom.replaceChildren(select.options[select.selectedIndex], document.createTextNode(newLabel));
			var indent = EditorUtil._getIndent(newLabel);
			if(indent > 0) {
				select.options[select.selectedIndex].style.paddingLeft= (indent * 0.4) + "em";
			} else {
				select.options[select.selectedIndex].style.paddingLeft="";
			}
			} catch(e) {};
		},
		_handleSelectBlur: function(e) {
			if (dojo.isIE) return;
			try{
			/* Special case for categories */
			if(!this._modifiers.allCategories) return;
			var select= EditorUtil._getOriginalTarget(e);
			for(var i=0;i<select.options.length;i++){
				if(select.options[i].__jazzWasClicked){
					dojox.data.dom.replaceChildren(select.options[i], document.createTextNode(select.options[i].__jazzLabel));
					var indent = EditorUtil._getIndent(select.options[i].__jazzLabel);
					if(indent > 0) {
						select.options[select.selectedIndex].style.paddingLeft= (indent * 0.4) + "em";
					} else {
						select.options[select.selectedIndex].style.paddingLeft="";
					}
					select.options[i].__jazzWasClicked = false;
				}
			}
			} catch(e) {};
		},
		_bindValuesToSelect: function(possibleValues, select, image) {
			var resultValue = "";
			var	selectedValueId  = select.value;
			if(selectedValueId == "") {
				selectedValueId=this.attributeValue ? this.attributeValue.id : "";
			}
			dojox.data.dom.removeChildren(select);
			
			if (!possibleValues || possibleValues.length === 0) {
				select.disabled= true;
				WidgetUtil.setSpacerImage(image);
				var label = this.messages.labelDefaultNullEnumerationValue;
				if (this._modifiers.noValueLabel != null) {
					label = this._modifiers.noValueLabel;
				}
				EditorUtil._addOption(select, new Option(label, EditorUtil.UNCONFIGURED));
				return resultValue;
			} else {
				select.disabled= this._isAlwaysDisabled;
			}
			
			var defaultIndex = 0;
			for (var i = 0; i < possibleValues.length; i++) {
				if(possibleValues[i].id == selectedValueId) {; // can be ""		
					defaultIndex= i;
					break;
				}
			}
			
			for (var i=0; i<possibleValues.length; i++) {
	
				var option= EditorUtil._createOption(possibleValues[i].label, possibleValues[i].id);
				option.valueObject = possibleValues[i];
				if(possibleValues[i].title) {
					option.title=possibleValues[i].title;
				}
				 
				option[EditorUtil._getLabelProp()]= possibleValues[i].label;
	
				var isDefault = i== defaultIndex;
	
				if(isDefault) {
					resultValue = possibleValues[i];
					select.currentDefaultIndex = i;
					if(possibleValues[i].iconUrl) {
						image.src = possibleValues[i].iconUrl;
						image.style.visibility="visible";	
					} else {
						WidgetUtil.setSpacerImage(image);
					}
				} 
				EditorUtil._addOption(select, option, isDefault);
				if(isDefault && possibleValues[i].title) {
					select.title = possibleValues[i].title;
				}
			
			}
			var label = "";
			try {
				label = this._getCurrentSelectProps(this._select).label;
			} catch(e) {}; // ignore
			this._setReadOnlySelectValue(label);
	//		if(moreLabel) {
	//			var option= this._createOption(moreLabel, moreId);
	//			option.title = moreTitle;
	//			this._addOption(select, option, false);
	//		}
		return resultValue;
	},
		getFormElementValues: function(formElements) {
//			if(!this._isChanged) return;
			if(this._select.disabled || this._isAlwaysDisabled) return;
			if (this._select.value != EditorUtil.UNCONFIGURED && (this._select.value != this.attributeValue.id)) {
				formElements[this.attributeId] = this._select.value;
			}
		},
	     getValue: function() {
		 	var value = this._select.value;
			return value;
		 },
		 getSelectedLabel: function() {
			if(!this._select) return "";
			return this._getCurrentSelectProps(this._select).label;
		},
		 setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
		 		if (this._select.value != retainedState[this.attributeId]) {
					this._select.value = retainedState[this.attributeId];
					this._isChanged = true;
//					this._setReadOnlySelectValue(this._getCurrentSelectProps(this._select).label);
					var selectValue = this._getCurrentSelectProps(this._select);
					this._setReadOnlySelectValue(selectValue.label);
					if(selectValue.iconUrl) {
						this._image.src = selectValue.iconUrl;
						this._image.style.visibility="visible";
					} else {
						WidgetUtil.setSpacerImage(this._image);
					}
				}
			}
		 },
		_getCurrentSelectProps: function(select) {
			if(select.selectedIndex == -1) return null;
			var currentOption= select.options[select.selectedIndex];
			var props= {
				label: currentOption[EditorUtil._getLabelProp()],
				id: select.value
			};
			if(currentOption.valueObject) {
				props.iconUrl = currentOption.valueObject.iconUrl;
			}
			return props;
		}
		
	}
);

})();
