/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.TimestampWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dijit.form.DateTextBox");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var util = com.ibm.team.workitem.web.client.util;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.TimestampWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="TimestampWidget">' +
						'<img dojoAttachPoint="_spacer" class="PreFieldImage" />' +
						'<span style="display: none" dojoAttachPoint="_dateReadOnlyValueSPAN" class="ReadOnlyTextValue TimestampLabel"></span>' +
						'<span style="display: inline" dojoAttachPoint="_dateTextSPAN" class="TimestampInput"><span dojoAttachPoint="_dateBoxSPAN"></span><img dojoAttachPoint="_calandarImage" class="CalendarImage" dojoAttachEvent="onclick: _handleCalendarClick"/></span>' +
						'</div>',
 		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			WidgetUtil.setSpacerImage(this._spacer);
			
			if(!this._modifiers) this._modifiers = {};			
			if(this.parentController.pageController.isGuest() || this._modifiers.readonly == "true") this._modifiers.readonly=true;
				
			var contributor = false;
			if(this._modifiers.readonly) {
				var value =  WidgetUtil.getFirstValue(this.values, true, true);
				if(this.presentation && this.presentation.properties) {
					for(var i=0;i<this.presentation.properties.length; i++) {
						if(this.presentation.properties[i] &&
							this.presentation.properties[i].key == "contributorAttributeId" ){
								var split = this.presentation.properties[i].value.split(".");
								if(split.length > 0) {
									var attrName = split[split.length-1];
									if(this.workItem && this.workItem.attributes && this.workItem.attributes[attrName]) {
										contributor = this.workItem.attributes[attrName];
									}
								}
							}
					}
				}
								
			} else {
				var value =  WidgetUtil.getFirstValue(this.values, true, true);
				if(value.content != null) value = value.content;
				if(value && !value.id && value.label) value = "";
				if(!value || (isNaN(value) && (value.indexOf("1969-12-31") != -1))) value = "";
			}

			this.createDateLabel(value, /* isSeconds */ false, contributor);
			if(!this._modifiers.readonly) {
				this.createDateTextBox();
			} else {
				this._dateTextSPAN.style.display="none";
				this._dateReadOnlyValueSPAN.style.display="inline";				
			}
			this._isChanged = false;
			this._calandarImage.src = ImagePool.CALENDAR;
		},
		destroy: function() {
			if(this.dateWidget) this.dateWidget.destroy();
		},
		setReadOnly: function(isReadOnly) {
			if(this._modifiers && this._modifiers.readonly)return;
			if (isReadOnly) { // was mouse out
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				this._dateTextSPAN.style.display="none";
				this._dateReadOnlyValueSPAN.style.display="inline";
			}
			else {
				if(this._modifiers && this._modifiers.readonly)return;
				this._dateTextSPAN.style.display="inline";
				this._dateReadOnlyValueSPAN.style.display="none";
			}
		},
		_handleCalendarClick: function(e) {
			this.dateWidget.focus();
		},
		createDateLabel: function(value, isSeconds, contributor) {
			var timeValue = value;
			
			if(value == "Unassigned") {
				value = "";
			}
			if(value.label && value.label == "Unassigned") {
				value.label = "";
			} 
			
			if(!value || value.length == 0) {
				if(this._modifiers.noValueLabel) {
					value = this._modifiers.noValueLabel; 
				} else {
					value = this.messages.labelDefaultNullDate;					
				}
				if (this._modifiers.noValueTitle) {
					timeValue = this._modifiers.noValueTitle;
				} else {
					timeValue = this.messages.titleNullDate;					
				}
				if (this._modifiers.readonly) {
					dojo.addClass(this._dateReadOnlyValueSPAN, "GreyLabel");
				}
			} else {
				var date;
				if(!isNaN(value) || !isSeconds) {
					date = util.fromISOString(value);					
				} else {
					date = new Date(value);
				}
				
				if(date != null ) {
					value = EditorUtil._getDateString(date, false);
					timeValue = EditorUtil._getDateString(date, true);
					// Defect 66967. Dojo does not return a string for all locales.
					if(!dojo.isString(timeValue)) {
						timeValue = value;
					}
					
					if(!contributor) {
						value = timeValue;
					}
				
				} else {
					if(value.label) {
						value = value.label;
					} else {
						if(value.id) {
							value = value.id
						} else {
							value = "";
						}
					}
					if (!value || value.length == 0) {
						if (this._modifiers.noValueLabel) {
							value = this._modifiers.noValueLabel;
						}
						else {
							value = this.messages.labelDefaultNullDate;
						}
						if (this._modifiers.noValueTitle) {
							timeValue = this._modifiers.noValueTitle;
						}
						else {
							timeValue = this.messages.titleNullDate;
						}
						if (this._modifiers.readonly) {
							dojo.addClass(this._dateReadOnlyValueSPAN, "GreyLabel");
						}
					}
				}
			}
			dojox.data.dom.removeChildren(this._dateReadOnlyValueSPAN);
			var dateSpan = document.createElement("SPAN");
			dateSpan.appendChild(document.createTextNode(value));
			// Defect 66967. Dojo does not return a string for all locales.
			if(dojo.isString(timeValue)) {
				dateSpan.title = timeValue;
			}			

 			this._dateReadOnlyValueSPAN.appendChild(dateSpan);
			var contribSpan = this.createContributorLabel(contributor);
			if(contribSpan) {
				this._dateReadOnlyValueSPAN.appendChild(contribSpan);	
			}
		},
		createContributorLabel: function (contributor) {
			if(!contributor || !contributor.name) return null;
			var href;
			if(contributor.emailAddress) {
				href = this.parentController._getMailto(contributor.emailAddress);
			}
// If there is an href, add the anchor, otherwise just add a label
// <a dojoAttachPoint="_anchor" class="LayoutValue Anchor"><span dojoAttachPoint="_labelSPAN" class=""></span></a>
			var labelTarget;
			if(href) {
				var anchor = document.createElement("A");
				anchor.href = href;
				dojo.addClass(anchor, "LayoutValue");
				dojo.addClass(anchor, "Anchor");
				anchor.style.paddingLeft = "1em;"
//				anchor.style.fontSize = "100%";
				anchor.appendChild(document.createTextNode(contributor.name));
				labelTarget = anchor;
			} else {
				labelTarget = document.createElement("SPAN");
				labelTarget.appendChild(document.createTextNode(contributor.name));
				labelTarget.className  = "LayoutContributorLabel";
			}
			
			return labelTarget;
		},
		createDateTextBox: function() {
		    dojo.addClass(this.domNode, "DateTextContainer"); 
			var value =  WidgetUtil.getFirstValue(this.values, true, true);
			if(value.content != null) value = value.content;
			if(value && !value.id && value.label) value = "";
			if(!value || (isNaN(value) && value.indexOf("1969-12-31") != -1)) value = "";
			var constraints = {};
			if(dojo.i18n.normalizeLocale() == "en-us") {
				constraints = { datePattern: "MMM d, yyyy"	}; 
			}
			this.dateWidget= /* createWidget */ new dijit.form.DateTextBox (
				{ isDate: true, containerToggle: "fade", constraints: constraints },
			this._dateBoxSPAN);
			var date = null;
			if (isNaN(value)) {
				if (value.length != 0) {
					date = util.fromISOString(value);
				}
			} else {
				date = new Date(value);
			}
			if(date)this.dateWidget.setValue(date);
			
// For some reason self does not resolve in the change function below
			this.dateWidget.__jazzWidget = this;
			var self = this;
			this.connect(this.dateWidget, "onChange", function(e) {
				self._handleDateChange();

//				var value = self.dateWidget.getValue().getTime();
//				self.currentValue = value;
//				self.createDateLabel(value, /*isSeconds*/ true);
			});
			
			// Date widget dynamically replaced its class so need to set element style.
			this.dateWidget.domNode.style.width = "71%";
			this.dateWidget.domNode.style.margin = "0";
			this.dateWidget.domNode.style.fontFamily = "inherit";
//			dojo.addClass(this.dateWidget.domNode, "DateInputField");
		},
		getFormElementValues: function(formElements) {
			if(this._modifiers.readonly) return;
//			if(!this._isChanged) return;
			var dueDate = null;
			try {
				dueDate = this.dateWidget.getValue();
				// Set to noon local
				dueDate.setHours(12);
				dueDate.setMinutes(0);		
				dueDate.setSeconds(0);
				dueDate.setMilliseconds(0);
				dueDate = dueDate.getTime();
			} catch(e) { } // ignore

			dueDate = dueDate === -1 ? null : dueDate;
			
			if(dueDate && (typeof dueDate) != "number") {
				var	dueDateDate = util.fromISOString(dueDate);
				if(dueDateDate) {
					dueDate =dueDateDate.getTime();						
				} else {
					dueDate = null;
				}
			}
			
			formElements[this.attributeId] = dueDate == null ? "" : {
				type: "date",
				value: dueDate
			};
			
		},
		setFormElementValues: function(retainedState){
			if(this._modifiers.readonly) return;

			if(retainedState[this.attributeId]) {
				if(!this.dateWidget)return;
				var currentValue = this.dateWidget.getValue() ? this.dateWidget.getValue().getTime() : null;
				if(currentValue != retainedState[this.attributeId]) {
					var value = retainedState[this.attributeId];
					if(value > 0) {
						this.dateWidget.setValue(new Date(value));
					} 
					this.createDateLabel(value, /*isSeconds*/ true);
					this._isChanged = true;
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , value);
					}
				}
			}
		},
		_handleDateChange: function() {
			this._isChanged=true;
			var e = { target: this.dateWidget };
			this.parentWidget._handleDirtyingEvent(e);
			var functionName = this._modifiers["onValueChanged"];
			if(functionName && this.parentWidget[functionName]) {
				var value = this.dateWidget.getValue()
				this.parentWidget[functionName](this, value);
			}
			this.createDateLabel(this.dateWidget.getDisplayedValue());
		}
	}
);

})();
