/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Columns"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var ImageClient			= com.ibm.team.apt.web.client.internal.ImageClient;
var WorkItemUtil		= com.ibm.team.workitem.web.client.util;

var ProgressBar			= com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

function _defaultAttributeColumnCreator(workItemWidget) {
	var result= document.createElement("td"); //$NON-NLS-1$
	dojo.addClass(result, "attribute"); //$NON-NLS-1$
	dojo.addClass(result, this._type);
	
	var icon= this._getIcon(workItemWidget);
	if (icon != null) {
		var iconElement= document.createElement("img"); //$NON-NLS-1$
		dojo.addClass(iconElement, "icon"); //$NON-NLS-1$
		iconElement.src= ImageClient.getImageUrl(icon);
		result.appendChild(iconElement);
	}

	var labelElement= document.createElement("span"); //$NON-NLS-1$
	dojo.addClass(labelElement, "label"); //$NON-NLS-1$
	labelElement.appendChild(document.createTextNode(this._getLabel(workItemWidget)));
	result.appendChild(labelElement);
	
	return result;
}

dojo.mixin(com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Columns,  {
	empty: {
		createCell: _defaultAttributeColumnCreator,
		
		_type: "empty", //$NON-NLS-1$
		_getIcon: function(workItemWidget) {
			return null;
		},
		_getLabel: function(workItemWidget) {
			return ""; //$NON-NLS-1$
		} 
	},

	summary: {
		createCell: function(workItemWidget) {
			var result= document.createElement("td"); //$NON-NLS-1$
			dojo.addClass(result, "attribute"); //$NON-NLS-1$
			dojo.addClass(result, "summary"); //$NON-NLS-1$
			
			var iconContainer= document.createElement("div"); //$NON-NLS-1$
			dojo.addClass(iconContainer, "icon"); //$NON-NLS-1$
			
			var iconElement= document.createElement("img"); //$NON-NLS-1$
			dojo.addClass(iconElement, "typeIcon"); //$NON-NLS-1$
			iconElement.src= ImageClient.getImageUrl(workItemWidget.getWorkItem().getWorkItemType().iconURL);
			iconContainer.appendChild(iconElement);

			var decorationElement= document.createElement("img"); //$NON-NLS-1$
			dojo.addClass(decorationElement, "typeDecoration"); //$NON-NLS-1$
			decorationElement.src= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/complete_ovr.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			iconContainer.appendChild(decorationElement);

			result.appendChild(iconContainer);
		
			var idLink= document.createElement("a"); //$NON-NLS-1$
			idLink.href= WorkItemUtil.getViewWorkItemURI(workItemWidget.getWorkItem().workItemId); 		
			dojo.addClass(idLink, "label"); //$NON-NLS-1$
			idLink.appendChild(document.createTextNode(workItemWidget.getWorkItem().summary));
			result.appendChild(idLink);
			
			return result;
		}
	},
	
	estimate: {
		createCell: _defaultAttributeColumnCreator,

		_type: "estimate", //$NON-NLS-1$
		_getIcon: function(workItemWidget) {
			return dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/duration_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		_getLabel: function(workItemWidget) {
			var label= workItemWidget.getWorkItem().estimate.label;
			return label.length > 0 ? label : "-"; //$NON-NLS-1$
		} 
	},
	
	priority: {
		createCell: _defaultAttributeColumnCreator,

		_type: "priority", //$NON-NLS-1$
		_getIcon: function(workItemWidget) {
			return workItemWidget.getWorkItem().getPriority().iconURL;
		},
		_getLabel: function(workItemWidget) {
			return workItemWidget.getWorkItem().getPriority().label;
		} 
	},
	
	owner: {
		createCell: _defaultAttributeColumnCreator,

		_type: "owner", //$NON-NLS-1$
		_getIcon: function(workItemWidget) {
			return workItemWidget.getWorkItem().getOwner().iconURL;
		},
		_getLabel: function(workItemWidget) {
			return workItemWidget.getWorkItem().getOwner().label;
		} 
	},

	progress: {
		createCell: _defaultAttributeColumnCreator,

		_type: "progress", //$NON-NLS-1$
		_getIcon: function(workItemWidget) {
			return null;
		},
		_getLabel: function(workItemWidget) {
			var workItem= workItemWidget.getWorkItem();
			
			var open= workItem.progressInformation.openCount;
			var closed= workItem.progressInformation.closeCount;
			
			if (open == 0 && closed == 0) {
				return MSGS.COLUMNS_PROGRESS_NOITEMS;
			}
			
			if (open == 0) {
				return MSGS.COLUMNS_PROGRESS_DONE;
			}

			return bind(MSGS.COLUMNS_PROGRESS_CLOSEDRATIO, [ Math.round((closed / (open + closed)) * 100) ]);
 		}
	},

	id: {
		createCell: function(workItemWidget) {
			var idColumn= document.createElement("td"); //$NON-NLS-1$
			dojo.addClass(idColumn, "id"); //$NON-NLS-1$

			var workItem= workItemWidget.getWorkItem();

			var idLink= document.createElement("a"); //$NON-NLS-1$
			idLink.href= WorkItemUtil.getViewWorkItemURI(workItem.workItemId); 		
			idLink.appendChild(document.createTextNode(workItem.workItemId));
			
			idColumn.appendChild(idLink);
			return idColumn;  		
		}
	}
});

})();
