/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.SearchResultSelector");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.JazzUtilHtml");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._base.popup");

dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function(){

var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var EditorUtil            = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var MAX_SIZE= 11;
var DELAY = 500;
    /* defineWidget */
    dojo.declare("com.ibm.team.workitem.web.ui.internal.view.query.SearchResultSelector", [dijit._Widget, dijit._Templated], {
    
        templateString: '<div class="com-ibm-team-workitem-web-search-selector">' +
        '<div dojoAttachPoint="_tagMatchArea"></div>' +
        '</div>',
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		},
        //dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/TagSelector.html"),
        // templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/TagSelector.css"),
        
        postCreate: function(args, frag){
            this.connect(this.domNode, "onclick", "_handleClick");
            this.connect(this._tagMatchArea, "onmouseover", "_handleMouseOver");
            this.connect(this._tagMatchArea, "onmouseout",  "_handleMouseOut");
			this._isEnabled=true;
			this._queryCount = 0;
			this._doneQueryCount = 0;
        },
        destroy: function() {
			this.close();
			this.inherited(arguments);
		},
		
		
        _handleClick: function(e){
            e.preventDefault();
            var target = e.target || e.srcElement;
            if (target.tagName === "A") {
                this._doSelect(target);
				this.parentWidget._handleSearch();
            }
        },
        
        _doSelect: function(link){
            if (!link._jazzTextIsMessage || link._jazzItem.isCommand) {
                this._callback(link._jazzItem);
				this._deferStart();
            }
            this.close();
        },
        
        _handleMouseOver: function(e){
            this.isMouseOver = true;
            var target = e.target || e.srcElement;
            if (target.tagName === "A") {
                this._highlightTag(target);
            }
        },
        _handleMouseOut: function(e){
            this.isMouseOver = false;
        },
        _deferStart: function() {
			this._deferStartTime = new Date().getTime();
		},
        open: function(){
            if (this.isShowingNow) {
                return;
            }
			if(this._deferStartTime  && (new Date().getTime()) - this._deferStartTime < 750) {
				return;
			}
			
            dijit.popup.open({
                popup: this,
                parent: this.parentWidget,
                around: this._tagNode
            });
            this._validateNode();
            this.isShowingNow = true;
        },
        showMatchingResults: function(searchTerm){
        
			if (this._currentId) {
				clearTimeout(this._currentId);
			}
			var self= this;
			var doSearchFunc= function() {
				self._doShowMatches(searchTerm);
			};
			this._currentId= setTimeout(doSearchFunc, DELAY);
		},
		
		_showLoading: function() {

			if (this.isShowingNow || (this._queryCount <= this._doneQueryCount)) return;
			if (this._tagNode && this._tagNode.value == "") {
				this.close();
				return;
			} 

			var item = {
                      value: this.messages.busyLoading,
                      isMessage: true,
					  isGrey: true
                    };
           this._showMatches([ item ], /* width */ "12.9em" );

		},
		_doShowMatches: function(searchTerm) {
			
          	if(!this._isEnabled || (this._deferStartTime  && (new Date().getTime()) - this._deferStartTime < 750)) {
				return;
			}
				
            if (dojo.trim(searchTerm) === "") {
                return;
            }
			
			setTimeout(dojo.hitch(this, this._showLoading), 500);
			this._queryCount++;
			var queryCount = this._queryCount;
			var self = this;
			var serviceObj= {
				success: function(summaryPage) {
					self._showQueryResults(summaryPage, queryCount);
				},
				failure: function(msg) {
					self._queryError(msg);
				}
			};

           var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			
            var params;
            if (WorkItemUtil.isWorkItemId(searchTerm)) 
                params = {
                    id: WorkItemUtil.getWorkItemId(searchTerm)
                };
            else {
				
                params = {
					maxResults: MAX_SIZE + 1,
                    fullText: searchTerm
                };
				if(this.parentWidget.openOnly) 
					params.status = "open";
			}
            this._setStatusMessage(this.messages.busyRunningSearch);
            QueryClient.getResults(params, srh);
            
            
        },
        _setStatusMessage: function(msg){
            // TODO: Show status msg
        },
        _queryError: function(msg){
			this._doneQueryCount++;
            console.debug("** ERROR ** " + msg);
        },
        _showQueryResults: function(summaryPage, queryCount){
			this._doneQueryCount++;			
			if ((this._queryCount != queryCount)) return;  // out of order
			if (this._tagNode && this._tagNode.value == "") {
				this.close();
				return;
			}

          	if(!this._isEnabled || (this._deferStartTime  && (new Date().getTime()) - this._deferStartTime < 750)) {
				return;
			}
		    var dtos = summaryPage ? summaryPage.workItemSummaryDTOs : [];
		    var dtos = dtos || [];
			var item, items = [];
			
			
            if (dtos.length === 0) {
                // if we have no matches or exactly one (case-sensitive) match, hide the tag picker
//                item = {
//					id: -1,
//                    value: "(No matches or not enough text for search)",
//                    isMessage: true
//                };
//                items.push(item);
//				this._showMatches(items);
           		this.close();
				return;
            }
			
			for(var i=0; i<Math.min(MAX_SIZE, dtos.length);i++ ){
				var value = WorkItemUtil.htmlToPlain(dtos[i].summary);
	
				if(value.length> 53) {
					value = value.substring(0,50) + "...";
				}
				
				item = {
					id: dtos[i].id,
					value:value,
					title:  dtos[i].id + ": " + value
				}	
				items.push(item);
			}
			
                // otherwise sort the results and show the matches
                items.sort(function(a, b){
                    return a.id < b.id ? -1 : 1;
                });
                if (dtos.length > MAX_SIZE) {
					var left = summaryPage.totalQueryResultCount - MAX_SIZE;
                    item = {
                        value: dojo.string.substitute(this.messages.labelSearchToSeeMore, [ ""+left ]),
                        isMessage: true
                    };
                    items.push(item);
                }
                
//				if (!WorkItemUtil.isWorkItemId(this.parentWidget._searchText.value) || items.length > 1) {
					item = {
						id: -1,
						value: this.parentWidget._queryType.title,
						isCommand: true
					};
					items.splice(0, 0, item);	
//				}
                
                this._showMatches(items);

        },
      
        setSearchResults: function(tagItemArr){
            if (!tagItemArr) {
                tagItemArr = [];
            }
            
            this._resultSet = tagItemArr;
            
        },
        
        highlightNextTag: function(direction){
            // if nothing is currently selected, just return
            
            var tags = this._tagMatchArea.getElementsByTagName("A");
            
            if (tags.length < 2) {
                return;
            }
            
            var curIndex = this._selectedTagIndex;
            if (direction === "up") {
                var newIndex = curIndex === 0 ? tags.length - 1 : curIndex - 1;
            }
            else {
                var newIndex = curIndex === tags.length - 1 ? 0 : curIndex + 1;
            }
            this._highlightTag(tags[newIndex]);
        },
        
        selectCurrentValue: function(){
            if (this._selectedTagIndex >= 0) {
                var tags = this._tagMatchArea.getElementsByTagName("A");
                this._doSelect(tags[this._selectedTagIndex]);
 				this._deferStart();
				this.enableDisplay(false);
            }
        },
        enableDisplay: function(isEnabled) {
			this._isEnabled = isEnabled;
		},
        close: function(){
            if (this._validateTimeout) {
                clearTimeout(this._validateTimeout)
                eval("delete this._validateTimeout");
            }
            if (this.isShowingNow) {
                dijit.popup.close(this);
                this.isShowingNow = false;
            }
        },
        _validateNode: function(){
            if (dojo.isDescendant(this._tagNode, document.body)) {
                this._validateTimeout = setTimeout(dojo.hitch(this, this._validateNode), 300);
                return;
            }
            this.close();
        },
        
        _showMatches: function(matchArr, width){
            dojox.data.dom.removeChildren(this._tagMatchArea);
			this.domNode.style.width = width == null ? "" : width; // can be null
            this._selectedTagIndex = -1;
            for (var i = 0; i < matchArr.length; i++) {
            
                var link = document.createElement("a");
                link.href = "search:" + matchArr[i];
				link.style.overflow = "hidden";
                var matchingPrefix = document.createElement("span");
                matchingPrefix.className = "MatchingPrefix";
				var sep;
                if (!matchArr[i].isMessage && !matchArr[i].isCommand) {
					sep = "";
					if(matchArr[i].id > 0) {
						sep = ": ";
                    	matchingPrefix.appendChild(document.createTextNode(matchArr[i].id + ""));
					}

                    var nonMatchingSuffix = document.createTextNode(sep + matchArr[i].value);
					link.title = matchArr[i].title;
                    link.appendChild(matchingPrefix);
                    link.appendChild(nonMatchingSuffix);
                }
                else {
                    var nonMatchingSuffix = document.createTextNode(matchArr[i].value);
					if(matchArr[i].isGrey) {
						link.style.color = "#AAAAAA";
					} 
                    link.appendChild(matchingPrefix);
                    link.appendChild(nonMatchingSuffix)
                    link._jazzTextIsMessage = true;
                }
                link._jazzItem = matchArr[i];
                this._tagMatchArea.appendChild(link);
                
                if (i === 0) {
                    this._highlightTag(link);
                }
            }
            
            // close and open the tag picker in case it went from being
            // positioned above an element to below an element (as sometimes
            // happens when the list goes from long to short on a smaller
            // screen)
            this.close();
            this.open();
        },
        
        _highlightTag: function(link){
            var tags = this._tagMatchArea.getElementsByTagName("A");
            for (var i = 0; i < tags.length; i++) {
                if (tags[i] === link) {
                    this._selectedTagIndex = i;
                    tags[i].className = "OnHover";
                }
                else {
                    tags[i].className = "";
                }
            }
        }
    });
    
    var SearchResultSelector = com.ibm.team.workitem.web.ui.internal.view.query.SearchResultSelector;
    
})();
