/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dijit.form.Button");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("com.ibm.team.workitem.web.client.internal.query.Term");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {
	
var Term= com.ibm.team.workitem.web.client.internal.query.Term;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/AttributeChooser.html"),
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textAddCondition = this.messages.labelAddCondition;
		this.attributeTree= args.attributeTree;
		this.termSection= args.termSection;
	},
	
	postCreate: function() {
		this._createAddTermButton();
		this._createTree();
		this._createAddAttributeButton();		
	},
	destroy: function() {
		if(this._tree) this._tree.destroy();
		if(this._addAttributeButton) this._addAttributeButton.destroy();
		
		this.inherited(arguments);
	},
	
	onClose: function() {
		this._contentNode.style.display= "none";
	},
	
	onAdd: function() {
		this._contentNode.style.display= "block";
	},
	
	_createTree: function() {
		var dataStore= this._createDataStore();
		var treeStore= new dojo.data.ItemFileReadStore({
			data: dataStore
		});
		var tree= new dijit.Tree({
			store: treeStore,
			widgetId: 'attributeTree',
			getLabelClass: function(item) {
				return (!item || item.isQueryable[0]) ? "" : "disabled";
			}
		});
		this._tree= tree;
		this._treeNode.appendChild(tree.domNode);
		this.connect(tree.domNode, "ondblclick", this._handleAddAttributeCondition);
	},
	
	_createAddTermButton: function() {
		var self= this;
		var operator= this.termSection.getOperator() == Term.AND ? Term.OR : Term.AND;
		var addTermButtonParameters= {
			label: operator == Term.AND ? self.messages.labelAddOperatorConditionAND : self.messages.labelAddOperatorConditionOR,
			onClick: function() {
				self._handleAddTermCondition(operator);
				self.onClose();
			}
		};
		var addTermButton= new dijit.form.Button(addTermButtonParameters);
		this._addTermButtonNode.appendChild(addTermButton.domNode);
	},
	
	_createAddAttributeButton: function() {
		var self= this;
		var addAttributeButtonParameters= {
			label: self.messages.labelAddAttributeCondition,
			onClick: function() {
				self._handleAddAttributeCondition();
			}
		};
		var addAttributeButton= new dijit.form.Button(addAttributeButtonParameters);
		this._addAttributeButton = addAttributeButton;
		this._addAttributeButtonNode.appendChild(addAttributeButton.domNode);
	},
	
	_handleAddAttributeCondition: function() {
		var item= this._getSelectedItem();
		if (item && item.isQueryable[0])
			this.termSection.addSection(item.id[0]); // the attribute identifier
	},
	
	_handleAddTermCondition: function(operator) {
		this.termSection.addTermSection({ operator: operator });
	},
	
	_getSelectedItem: function() {
		var tree= this._tree;
		var lastFocusNode= tree.lastFocused;
		if (lastFocusNode == null) {
			return null;
		}
		return lastFocusNode.item;
	},
	
	_createDataStore: function() {
		var store= new Object();
		store.label= "name";
		store.identifier= "id";
		store.items= this._createItems(this.attributeTree.children);
		return store;
	},
	
	_createItems: function(dtos){
		var items= new Array();
		for (var i= 0; i < dtos.length; i++) {
			var attribute= dtos[i].item;
			var item= {
				name: attribute.label,
				id: [attribute.id],
				isQueryable: attribute.isQueryable
			};
			if (dtos[i].children) {
				item.children= this._createItems(dtos[i].children);
			}
			if (attribute.isQueryable || item.children)
				items.push(item);
		}
		items.sort(this._sortAttributes);
		return items;
	},
	
	_sortAttributes: function(entry1, entry2){
		if (entry1.name < entry2.name) {
			return -1;
		}
		else if (entry1.name > entry2.name) {
			return 1;
		}
		return 0;
	}
});
	
})();
